﻿/**
RA Approval controls
**/
define(function (require) {
    var tpl = require("text!forms/header/approval/view.html");

    var definition = {
        template: tpl,
        build: function (vm, node, callback) {
            //build template using underscore.js so that we can interpret kendo template vars if needed
            var built = _.template(tpl);
            var rejected = "107fc6bd-2bb3-0282-4425-f43b5b32ef13";
            var approve = "0e856c6c-04e5-0a8e-6041-bc7715b4747e";
            var notYetVoted = "dae75d12-89ac-a8d8-4fe3-516c2a6a26f7";


            vm.view.approvalModel = new kendo.observable({
                decisionEnumId: null,
                comments: "",
                onRejectClick: function () {
                    vm.view.approvalModel.decisionEnumId = rejected;
                    this.updateStatus();
                },
                onApproveClick: function () {
                    vm.view.approvalModel.decisionEnumId = approve;

                    this.updateStatus();
                },
                onCompleteClick: function () {
                    var status = { Id: app.constants.workItemStatuses.ManualActivity.Completed, Name: localization.Completed };
                    vm.view.approvalModel.updateManualActivity(status);
                },
                onFailClick: function () {
                    var status = { Id: app.constants.workItemStatuses.ManualActivity.Failed, Name: localization.Failed }
                    vm.view.approvalModel.updateManualActivity(status);
                },
                updateStatus: function () {
                    if (vm.view.approvalModel.isReviewActivity) {
                        vm.view.approvalModel.updateReviewActivity();
                    }

                    if (vm.view.approvalModel.isManualActivity) {
                        vm.view.approvalModel.updateManualActivity();
                    }
                },
                updateReviewActivity: function () {
                    $.when(kendo.ui.ExtYesNoTextAreaDialog.show({
                        title: localization.Comments,
                        message: localization.DoYouHaveComments,
                        required: true,
                        height: 250
                    })
                  ).done(function (response) {
                      if (response.button === "yes") {
                          vm.view.approvalModel.comments = response.input;
                      }


                      _.each(pageForm.viewModel.Reviewer, function (reviewItem) {

                          var group = _.find(pageForm.userGroup, function (group) {
                              return (!_.isUndefined(reviewItem.User)) ? reviewItem.User.BaseId == group.Id : false;
                          });

                          if (reviewItem.Decision.Id == notYetVoted
                              && ((!_.isUndefined(reviewItem.User) && reviewItem.User.BaseId == session.user.Id)
                              || !_.isUndefined(group))) {

                              var comment = "";
                              if (_.isNull(reviewItem.Comments)) {
                                  comment = vm.view.approvalModel.comments;
                              }
                              else {
                                  comment = reviewItem.Comments + " " + vm.view.approvalModel.comments;
                              }

                              reviewItem.VotedBy = {};
                              reviewItem.VotedBy.BaseId = session.user.Id;
                              reviewItem.Decision.set("Id", vm.view.approvalModel.decisionEnumId);
                              reviewItem.set("Comments", comment);
                              reviewItem.set("DecisionDate", new Date().toISOString());

                          }
                      });

                      //exclude parent workitem details
                      if (pageForm.viewModel.ParentWorkItem) delete pageForm.viewModel.ParentWorkItem;

                      vm.view.approvalModel.save();
                  });
                },
                updateManualActivity: function (status) {
                    $.when(kendo.ui.ExtYesNoTextAreaDialog.show({
                        title: session.consoleSetting.RequireImplementationNotes ? localization.ImplementationdetailsRequired : localization.Implementationdetails,
                        buttons: [
                            {
                                name: localization.Save,
                                click: function (e) {
                                    var $inputText = $("#extInputDialog .k-ext-input-dialog-input");
                                    var comment = $("#extInputDialog .k-ext-input-dialog-input").val();
                                    var currentNotes = !_.isNull(pageForm.viewModel.Notes) ? pageForm.viewModel.Notes : "";

                                    if (e.dialog.options.required && $inputText.val().length == 0) {
                                        $inputText.addClass(e.dialog.options.requiredCss);
                                    } else {
                                        //exclude parent workitem details
                                        if (pageForm.viewModel.ParentWorkItem) delete pageForm.viewModel.ParentWorkItem;

                                        //set status 
                                        pageForm.viewModel.set("Status", status)

                                        //set notes
                                        pageForm.viewModel.set("Notes", currentNotes + "\n" + comment);
                                        vm.view.approvalModel.save();
                                        e.dialog.close();
                                    }
                                },
                                cls: "btn-primary"
                            },
                            {
                                name: localization.Cancel,
                                click: function (e) {
                                    e.dialog.close();
                                }
                        }],
                        required: session.consoleSetting.RequireImplementationNotes,
                        requiredCss: "k-ext-required"
                    })
                 );
                },
                save: function () {
                    var postData = encodeURIComponent(JSON.stringify({
                        isDirty: true,
                        current: pageForm.viewModel.toJSON(),
                        original: pageForm.jsonRaw
                    }));

                    $.ajax({
                        type: 'POST',
                        dataType: 'text',
                        url: pageForm.saveUrl,
                        data: "formJson=" + postData,
                        success: function (data, status, xhr) {
                            location.reload();
                            if (data.search('loginForm') < 0) { // Logged out check    
                                var successMessage = vm.view.approvalModel.decisionEnumId == "0e856c6c-04e5-0a8e-6041-bc7715b4747e" ?
                                    localization.ReviewActivityApprovedMessage : localization.ReviewActivityRejectedMessage;
                                
                                if (pageForm.viewModel.ClassName == "System.WorkItem.Activity.ManualActivity") {
                                    successMessage = (pageForm.viewModel.Status.Id == app.constants.workItemStatuses.ManualActivity.Completed) ?
                                                    localization.ManualActivityCompletedMessage :
                                                    localization.ManualActivityFailedMessage;
                                }

                                app.lib.message.add(successMessage.replace("{0}", vm.viewModel.Id), "success");
                                app.lib.handleMessages();
                            } else {
                                //session expired
                                window.location = "/Login/Login?ReturnUrl=" + window.location.pathname;
                            }
                        },
                        error: function (xhr, ajaxOptions, thrownError) {
                            //do we have a data conflict
                            if (xhr.status == 409) {

                                $.when(kendo.ui.ExtYesNoDialog.show({
                                    title: localization.DataConflict,
                                    message: localization.DataConflictError + "<br/><br/><small>(" + localization.DataConflictDescription + ")</small>",
                                    icon: "fa fa-exchange text-danger",
                                    width: "500px",
                                    height: "175px"
                                })
                                ).done(function (response) {
                                    if (response.button === "yes") {
                                        //open in new tab, may not work this way in all browsers
                                        window.open(window.location.pathname);
                                    } else {
                                        //refresh page
                                        location.href = window.location.pathname;
                                    }
                                });

                            } else if (xhr.status == 503) { //SDK unavailable
                                var jsonRsp = xhr.responseText;
                                app.lib.log(jsonRsp);
                                //determine error Message
                                var errorMsg = localization.ReconnectRequired;
                                if (jsonRsp.exception && jsonRsp.exception.length > 0) {
                                    errorMsg = jsonRsp.exception;
                                }
                                //alert the user
                                kendo.ui.ExtAlertDialog.show({
                                    title: localization.Failed,
                                    message: errorMsg,
                                    icon: "fa fa-times-circle text-danger"
                                });
                            } else if (xhr.status == 403) { //user does not have access
                                var jsonRsp = xhr.responseText;
                                app.lib.log(jsonRsp);
                                //determine error Message
                                var errorMsg = localization.RequestFailed;
                                if (jsonRsp.exception && jsonRsp.exception.length > 0) {
                                    errorMsg = jsonRsp.exception;
                                }
                                //alert the user
                                kendo.ui.ExtAlertDialog.show({
                                    title: localization.Failed,
                                    message: errorMsg,
                                    icon: "fa fa-times-circle text-danger"
                                });

                            } else {
                                console && app.lib.log(localization.RequestFailed);
                                app.lib.log(thrownError);

                                var jsonRsp = xhr.responseText;
                                app.lib.log(jsonRsp);

                                var errorMsg = localization.RequestFailed;
                                if (jsonRsp.exception && jsonRsp.exception.length > 0) {
                                    errorMsg = jsonRsp.exception;
                                }

                                kendo.ui.ExtAlertDialog.show({
                                    title: localization.ErrorDescription,
                                    message: errorMsg,
                                    icon: "fa fa-exclamation"
                                });
                            }
                        },
                        processData: false,
                        async: false
                    });
                },
                pageDataSource: new kendo.data.DataSource({
                    serverFiltering: true,
                    serverSorting: true,
                    transport: {
                        read: {
                            url: pageForm.viewModel.ClassTypeId.toLowerCase() == "bfd90aaa-80dd-0fbb-6eaf-65d92c1d8e36" ? "/api/V3/WorkItem/GetGridWorkItemsMyApprovals/" : "/api/V3/WorkItem/GetGridWorkItemsMyManualActivities/",
                            data: {
                                userId: session.user.Id,
                                isScoped: session.user.Security.IsWorkItemScoped,
                                showInActives: false
                            },
                            cache: false
                        }
                    },
                    requestEnd: function (e) {
                        vm.view.approvalModel.updatePageDetails(e.response);
                    }
                }),
                updatePageDetails: function (data) {
                    var index = 1;
                    var pageCount = (data) ? data.length : 0;

                    _.find(data, function (item, idx) {
                        if (item.Id == vm.viewModel.Id) {
                            index = idx;
                        }
                    });

                    var pageString = (index + 1) + " / " + pageCount;

                    vm.view.approvalModel.set("currentPage", index);
                    vm.view.approvalModel.set("toatPage", pageCount);
                    vm.view.approvalModel.set("pageText", pageString);
                    vm.view.approvalModel.set("disablePrevious", (index == 0) ? true : false);
                    vm.view.approvalModel.set("disableNext", (index == pageCount - 1) ? true : false);

                    var statusInProgress = vm.viewModel.Status.Id === "11fc3cef-15e5-bca4-dee0-9c1155ec8d83";

                    if (!statusInProgress || (!_.isUndefined(vm.viewModel.reviewDetails) && (vm.viewModel.reviewDetails.fullDetails.Decision.Id == approve || vm.viewModel.reviewDetails.fullDetails.Decision.Id == rejected) && (vm.viewModel.reviewDetails.fullDetails.User.BaseId == session.user.Id))) {
                        vm.view.approvalModel.set("disabledButtons", true);
                    }
                    else {
                        var review = _.find(pageForm.viewModel.Reviewer, function (reviewItem) {
                            if (_.isUndefined(reviewItem.User)) {
                                return null;
                            }
                            var group = _.find(pageForm.userGroup, function (group) {
                                return reviewItem.User.BaseId == group.Id;
                            });

                            return !_.isUndefined(reviewItem.User) && reviewItem.Decision.Id == notYetVoted && (reviewItem.User.BaseId == session.user.Id || !_.isUndefined(group));
                        });

                        if (!review) return;

                        if (!statusInProgress || (_.isUndefined(review) || _.isNull(review))) {
                            vm.view.approvalModel.set("disabledButtons", true);
                        }
                    }
                },
                nextClick: function () {
                    var index = vm.view.approvalModel.get("currentPage") + 1;
                    vm.view.approvalModel.navigate(index);
                },
                previousClick: function () {
                    var index = vm.view.approvalModel.get("currentPage") - 1;
                    vm.view.approvalModel.navigate(index);
                },
                navigate: function (index) {
                    vm.view.approvalModel.pageDataSource.read();
                    var dSource = vm.view.approvalModel.pageDataSource.data();

                    var nextItem = dSource[index];
                    var url = vm.view.approvalModel.isReviewActivity ? "/ReviewActivity/Approval/" : "/ManualActivity/Complete/";
                    var approvalUrl = url + nextItem.Id;

                    location.href = approvalUrl;
                },
                onChangeSupportGroup: function() {
                    vm.viewModel[pageForm.MASupportGroupField] = vm.view.approvalModel.SupportGroupFieldValue;
                },
                currentPage: 0,
                toatPage: 0,
                pageText: "",
                disablePrevious: false,
                disableNext: false,
                title: vm.view.title,
                workItemId: vm.viewModel.Id,
                statusVisible: vm.view.statusVisible,
                statusName: vm.viewModel.Status.Name,
                isParent: vm.viewModel.IsParent,
                isDesktopView: !app.isMobile(),
                isMobileView: app.isMobile(),
                disabledButtons: false,
                reviewResult: null,
                reviewId: 0,
                SupportGroupVisible: pageForm.MASupportGroupField !== "",
                SupportGroupEnumId: pageForm.MASupportGroupGuid,
                SupportGroupFieldValue: vm.viewModel[pageForm.MASupportGroupField],
                isManualActivity: pageForm.viewModel.ClassName == "System.WorkItem.Activity.ManualActivity",
                isReviewActivity: pageForm.viewModel.ClassName == "System.WorkItem.Activity.ReviewActivity",
                disabledMAButtons: pageForm.viewModel.Status.Id == app.constants.workItemStatuses.ManualActivity.Completed ||
                    pageForm.viewModel.Status.Id == app.constants.workItemStatuses.ManualActivity.Failed
                
            });

            vm.view.approvalModel.pageDataSource.read();
            var view = new kendo.View(built(), { wrap: false, model: vm.view.approvalModel });

            callback(view.render());


        }
    }
    
    return definition;

});