﻿/**
multipleObjectPicker
**/

define(function (require) {
    var tpl = require("text!forms/predefined/multipleObjectPicker/view.html");
    var objectPickerPopup = require("forms/popups/multipleObjectPickerPopup/controller");

    var projectionSearchUrl = "/Search/GetObjectProjectionByCriteria";
    var definition = {
        template: tpl,
        build: function (vm, node, callback) {
            boundObj = node.vm;
            name = node.PropertyName;

            //check if string can be localized
            if (!_.isUndefined(localization[node.name]) && [node.name].length > 0) {
                node.name = localization[node.name];
            }

            var properties = {
                Disabled: false,
                ClassId: "",
                PropertyToDisplay: [],
                PropertyName: "",
                ShowAddButton: true,
                ShowRemoveButton: true,
                Scoped: false,
                ClassProperties: "",
                SelectableRow: false,
                SelectProperty: "Id",
                ProjectionId: "",
                HiddenProperty: null,//separated by comma only
                IsMobileView: app.isMobileDevice()
            };

            $.extend(true, properties, node);

            if (properties.PropertyToDisplay.length <= 0) { properties.PropertyToDisplay = { DisplayName: "DisplayName", Path: "Path", Status: "Status" }; }
            for (var prop in properties.PropertyToDisplay) {
                properties.ClassProperties = properties.ClassProperties.concat(prop).concat(",");
            }

            if ((properties.PropertyName === "HasRelatedWorkItems") && (properties.ClassProperties.indexOf("AssetStatus") < 0)) {
                properties.ClassProperties = properties.ClassProperties.concat("AssetStatus");
            }

            if (!_.isNull(properties.HiddenProperty)) { //This will be use to load properties that is not need to display
                properties.ClassProperties += properties.HiddenProperty.concat(",");
            }

            //set properties for watchlist
            if ((properties.PropertyName === "WatchList")) {
                if (!session.consoleSetting.DashboardsLicense.IsValid || !session.enableWatchlist) {
                    properties.Visible = false;
                }
                //hide add/remove option when current user is and enduser and he or she is not the affected user
                if ((!session.user.Analyst) && (typeof boundObj.RequestedWorkItem != 'undefined') && (session.user.Id != boundObj.RequestedWorkItem.BaseId)) {
                    properties.ShowAddButton = false;
                    properties.ShowRemoveButton = false;
                }
            }

            var built = _.template(tpl);
            var templateFrag = $(built(properties));

            //used on RelatesToWorkItem relatinship, exclude current workitem from list
            var filter = { field: "BaseId", operator: "neq", value: boundObj.BaseId };

            var popupWindow = (properties.PropertyName === "RelatesToWorkItem") ?
                objectPickerPopup.getPopup(properties.ClassId, properties.ClassProperties, null, filter) :
                objectPickerPopup.getPopup(properties.ClassId, properties.ClassProperties);

            var objectPicker = new Control(templateFrag.find("[data-control='multipleObjectPicker']"), boundObj, name, properties, popupWindow);
            templateFrag.find("[data-control='multipleObjectPicker']").attr("data-control", "multipleObjectPickerBound");

            callback(templateFrag);
        },
        openObject: function (selectedItem) {
            var strClass = selectedItem.ClassName.split(/[\s.]+/);
            var className = strClass[strClass.length - 1];
            switch (className) {
                case "HardwareAsset":
                case "SoftwareAsset":
                    window.open("/AssetManagement/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Lease":
                case "Warranty":
                    window.open("/AssetManagement/Contract/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "SupportContract":
                    window.open("/AssetManagement/Contract/SupportandMaintenance/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Location":
                case "NoticeEvent":
                case "Organization":
                case "Standard":
                case "Subnet":
                case "Vendor":
                case "CatalogItem":
                case "CostCenter":
                case "Invoice":
                case "PurchaseOrder":
                case "Purchase":
                    window.open("/AssetManagement/Administration/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Licence":
                    window.open("/AssetManagement/Administration/License/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Consumable":
                    window.open("/AssetManagement/Administration/Consumables/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "ManualActivity":
                case "ReviewActivity":
                case "SequentialActivity":
                case "ParallelActivity":
                case "DependentActivity":
                    window.open("/Activity/Edit/" + selectedItem.Id, '_blank');
                    break;
                case "Incident":
                case "ServiceRequest":
                case "ChangeRequest":
                case "Problem":
                case "ReleaseRecord":
                    window.open("/" + className + "/Edit/" + selectedItem.Id + "/", '_blank');
                    break;
                default:
                    break;
            }
        }
    }

    var Control = function (targetEle, boundObj, name, properties, popupWindow) {

        //use setters and getters if you want vm boundOdj to trigger change
        if (_.isUndefined(boundObj[name])) {
            boundObj.set(name, new kendo.data.ObservableArray([]));
        }

        //if related object is of single cardinality, create a new array and add the related object 
        var boundArray = !_.isUndefined(boundObj[name].length) ? boundObj.get(name) : new kendo.data.ObservableArray([boundObj.get(name)]);
        boundArray.properties = properties;

        var self = this;

        var __construct = function () {
            buildSubProperties();
            bindButtonEvents();
            createGrid();
        }

        var buildSubProperties = function () {
            var propToFill = [];
            //This will going to build array of display properties that has sub properties. Ex:Target_PurchaseHasPurchaseOrder.DisplayName
            for (var prop in properties.PropertyToDisplay) {
                var propSplit = prop.split(".");
                if (propSplit.length > 1) {
                    propToFill.push(propSplit);
                }
            }

            //This will going to check all properties. If property is not existing, this will going to create a dummy property so that kendo will render the grid.
            _.each(boundArray, function (data) {
                _.each(propToFill, function (prop) {
                    var mainProp = prop[0];
                    var subProp = prop[1];
                    if (_.isUndefined(data[mainProp])) {
                        data[mainProp] = {};
                        data[mainProp][subProp] = "";
                    }
                });
            });
        }

        //Make this global so that this can be used when inserting data outside multiple object picker.
        //This is the solution for now since kendo grid doesn't have preDataBinding which I need to manipulate data before binding
        //This function is called also from software asset "copy license assignment" task
        boundArray.addItem = function (object) {
            var isItemExist = false;
            if (isDuplicate(object.BaseId)) {
                return;
            }
            _boundArray = this;
            //This will going to get object related items
            if (!_.isUndefined(_boundArray.properties.ProjectionId) && !_.isNull(_boundArray.properties.ProjectionId) && _boundArray.properties.ProjectionId != "") {

                var colNames = "";
                var splitedProp = [];
                //This will going to get thos property whos have sub child on it.
                //Ex. PropertyToDisplay: { DisplayName: "DisplayName", "Currency.Name": "Currency", Cost: "Cost","Target_PurchaseHasPurchaseOrder.DisplayName":"Purchase Order" }
                // => Target_PurchaseHasPurchaseOrder.DisplayName and Currency.Name
                for (var prop in _boundArray.properties.PropertyToDisplay) {
                    var propperty = prop.split(".");
                    if (propperty.length > 1) {
                        splitedProp.push(propperty);
                        colNames = colNames == "" ? prop : colNames + "," + prop;

                    }
                }

                $.post(projectionSearchUrl, { baseId: object.BaseId, projectionId: _boundArray.properties.ProjectionId, columnNames: colNames }, function (result) {
                    for (index in result.Data) {
                        for (propIndex in splitedProp) {
                            var value = result.Data[index][splitedProp[propIndex][0]];
                            if (!_.isUndefined(value[splitedProp[propIndex][1]])) { //this will check if this property is undefined or not. This will going to avoid conflict with enums
                                object[splitedProp[propIndex][0]] = value;
                            }
                        }
                    }

                    _boundArray.push(object);
                });
            } else {
                _boundArray.push(object);
            }
        }

        var bindButtonEvents = function () {
            var actions = {
                addClick: function () {
                    popupWindow.setSaveCallback(function (object) {
                        boundArray.addItem(object);

                        //if related object is of single cardinality, replace the value
                        if (_.isUndefined(boundObj[name].length)) {
                            boundArray.shift();
                            boundObj.set(name, boundArray[0]);
                        }
                    });
                    popupWindow.open();
                }
            };

            //bind add click event
            targetEle.find('[data-click]').each(function () {
                if (actions[$(this).attr('data-click')]) {
                    $(this).click(actions[$(this).attr('data-click')]);
                }
            });

            //hide add button when ShowAddButton=false or if Disabled
            if (!properties.ShowAddButton || properties.Disabled || properties.vm.isDisabled) {
                targetEle.find('[data-click]').each(function () {
                    $(this).hide();
                });
            }
        }

        var createGrid = function () {
            var gridEle = targetEle.find("[data-control-grid]");
            //always unhide remove button for watchlist 
            var hideRemoveButton = (properties.PropertyName === "WatchList") ? false : (!properties.ShowRemoveButton || properties.Disabled || properties.vm.isDisabled);
            //building grid columns
            var columnFields = buildColumnFields(properties.PropertyToDisplay);
            var columnToDisplay = buildGridColumns(columnFields, hideRemoveButton);

            var grid = gridEle.kendoGrid({
                columns: columnToDisplay,
                scrollable: false,
                filterable: {
                    extra: false,
                    messages: {
                        info: localization.Showitemswithvaluethat,
                        and: localization.And,
                        or: localization.Or,
                        filter: localization.Filter,
                        clear: localization.Clear
                    },
                    operators: {
                        string: {
                            startswith: localization.Startswith,
                            contains: localization.Contains,
                            eq: localization.Isequalto,
                            neq: localization.Isnotequalto
                        }
                    }
                },
                reorderable: true,
                sortable: true,
                columnMenu: {
                    messages: {
                        columns: localization.ChooseColumns,
                        filter: localization.Filter,
                        sortAscending: localization.SortAscending,
                        sortDescending: localization.SortDescending
                    }
                },
                editable: {
                    confirmDelete: "Delete",
                    cancelDelete: "Cancel",
                    mode: "incell",
                    template: null,
                    create: true,
                    update: false,
                    destroy: true
                },
                toolbar: {},
                dataSource: {
                    transport: {
                        prefix: "",
                        read: {
                            url: ""
                        }
                    },
                    type: "aspnetmvc-ajax",
                    schema: {
                        data: "Data",
                        total: "Total",
                        errors: "Errors",
                        model: {
                            id: "BaseId",
                            fields: columnFields
                        }
                    },
                    data: {
                        Data: boundArray,
                        Total: boundArray.length
                    }
                },
                dataBinding: function (e) {
                    //Change to only pass specific item
                    if (!_.isUndefined(e.items) && e.items.length > 0) {
                        formatPropertyInBoundArray(e, properties.ClassProperties)
                    }
                },
                dataBound: function (e) {
                    if (properties.Scoped) {
                        scopeBoundArray(this, properties.ClassId, properties.ClassProperties);
                    }

                },
                selectable: properties.SelectableRow ? "multiple row" : false
            });

            gridEle.on("click", "td", function (e) {
                var highlightedColumn = $(e.currentTarget).hasClass("grid-highlight-column");
                if (!highlightedColumn) { return }

                var kendoGrid = grid.data("kendoGrid");
                var selectedItem = kendoGrid.dataItem($(e.currentTarget).closest("tr"));
                openObject(selectedItem);
            });

            if (!app.isMobile())
            {
                grid.data("kendoGrid").dataSource.originalFilter = grid.data("kendoGrid").dataSource.filter;

                // Replace the original filter function.
                grid.data("kendoGrid").dataSource.filter = function () {

                    // Call the original filter function.
                    var result = grid.data("kendoGrid").dataSource.originalFilter.apply(this, arguments);

                    // If a column is about to be filtered, then raise a new "filtering" event.
                    if (arguments.length > 0) {
                        this.trigger("filterApplied", arguments);
                    }

                    return result;
                }

                grid.data("kendoGrid").dataSource.bind("filterApplied", function () {
                    $.each($(grid).find("th a.k-header-column-menu i"), function () {
                        $(this).remove();
                    });

                    if (grid.data("kendoGrid").dataSource.filter()) {

                        var dsFilters = grid.data("kendoGrid").dataSource.filter().filters || [];

                        for (var i in dsFilters) {
                            $(grid).find("th[data-field=" + dsFilters[i].field + "] a.k-header-column-menu").append("<i class=\"fa fa-filter\"></i>");
                        }

                    }
                });
            }

            

        }

        var isDuplicate = function (idToAdd) {
            var n = false;
            $.each(boundArray, function (i, item) {
                if (item.BaseId == idToAdd) {
                    n = true;
                }
            });
            return n;
        }

        var buildGridColumns = function (columnFields, showRemovButton) {
            var columns = [];

            for (var prop in properties.PropertyToDisplay) {
                var propertyKey = prop;
                var localizationKey = properties.PropertyToDisplay[prop];
                var localizedText = (!_.isUndefined(localization[localizationKey]) && [localizationKey].length > 0) ? localization[localizationKey] : localizationKey;
                var columnClass = (properties.SelectableRow && propertyKey === properties.SelectProperty) ? "grid-highlight-column" : "";
                var dateFormat = (columnFields[propertyKey].type == "date") ? "{0:" + kendo.culture().calendar.patterns.g + "}" : null;

                columns.push({
                    title: localizedText,
                    width: "210px",
                    field: propertyKey,
                    filterable: {},
                    encoded: true,
                    attributes: {
                        "class": columnClass
                    },
                    format: dateFormat
                });
            }
            columns.push({ title: localization.BaseId, width: "210px", field: "BaseId", filterable: {}, encoded: true, hidden: true });
            columns.push({ width: "105px", command: [{ name: "destroy", text: localization.Remove }], hidden: showRemovButton });

            return columns;
        }


        //traverse and format date
        var formatPropertyInBoundArray = function (currentItemEvent, columnString) {
            var currentItem = currentItemEvent.items[0];
            $.each(columnString.split(","), function (i, column) {
                $.each(boundArray, function (index, item) {
                    //mutate poorly structured provided enum data 
                    if (column.indexOf('.') > 0) {
                        var property = column.split('.')[0];
                        if (_.isUndefined(item[property])) {
                            boundArray[index][property] = { Id: "", Name: "" }
                        }
                    }
                    if (currentItem.BaseId != item.BaseId && currentItemEvent.action == "add") return; //This will be only used when adding new items
                    if (_.isUndefined(item[column]) || _.isNull(item[column])) { return; }
                    if (_.isBoolean(item[column]) || _.isNumber(item[column])) { return; }
                    if (!_.isUndefined(item[column].Name)) { boundArray[index][column] = item[column].Name; } //enums
                });
            });
        }

        var scopeBoundArray = function (grid, scopeClassId, PropertyToDisplay) {

            if (scopeClassId === "") { return; }

            $.ajax({
                url: "/Search/GetSearchObjectsByClassId",
                data: { searchFilter: "", classId: scopeClassId, columnNames: PropertyToDisplay },
                async: false,
                timeout: 3000,
                success: function (data) {
                    var objectIds = [];
                    $.each(data.Data, function (ii, iitem) {
                        objectIds.push(iitem["BaseId"]);
                    });

                    $.each(grid._data, function (ii, iitem) {
                        if (!_.contains(objectIds, this.BaseId)) {
                            var row = grid.tbody.find("tr[data-uid='" + this.uid + "']");
                            row.hide();
                        }
                    });
                }
            });
        }

        var openObject = function (selectedItem) {
            var strClass = selectedItem.ClassName.split(/[\s.]+/);
            var className = strClass[strClass.length - 1];
            switch (className) {
                case "HardwareAsset":
                case "SoftwareAsset":
                    window.open("/AssetManagement/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Lease":
                case "Warranty":
                    window.open("/AssetManagement/Contract/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "SupportContract":
                    window.open("/AssetManagement/Contract/SupportandMaintenance/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Location":
                case "NoticeEvent":
                case "Organization":
                case "Standard":
                case "Subnet":
                case "Vendor":
                case "CatalogItem":
                case "CostCenter":
                case "Invoice":
                case "PurchaseOrder":
                case "Purchase":
                    window.open("/AssetManagement/Administration/" + className + "/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Licence":
                    window.open("/AssetManagement/Administration/License/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "Consumable":
                    window.open("/AssetManagement/Administration/Consumables/Edit/" + selectedItem.BaseId + "/", '_blank');
                    break;
                case "ManualActivity":
                case "ReviewActivity":
                case "SequentialActivity":
                case "ParallelActivity":
                case "DependentActivity":
                    window.open("/Activity/Edit/" + selectedItem.Id, '_blank');
                    break;
                case "Incident":
                case "ServiceRequest":
                case "ChangeRequest":
                case "Problem":
                case "ReleaseRecord":
                    window.open("/" + className + "/Edit/" + selectedItem.Id + "/", '_blank');
                    break;
                default:
                    break;
            }
        }

        var buildColumnFields = function (properties) {
            var stringProp;

            stringProp = '{';
            for (var prop in properties) {
                var type = (prop.indexOf("Date") > -1 || prop == "LastModified") ? "date" : "string";
                stringProp += '"' + prop + '":{"type":"' + type + '"},';
            }
            stringProp = stringProp.replace(/,([^,]*)$/, '$1');
            stringProp += '}';

            return JSON.parse(stringProp);
        }
        __construct();
    }
    return definition;

});