﻿/**
Source Fields: gets the dataSource.schema.model.fields data based on .net Class type
**/
define(function (require) {

    var definition = {
        build: function (vm, node, callback) {
            vm.dataSource.schema.model.fields = {}
            _.each(vm.grid.columns, function (col) {
                vm.dataSource.schema.model.fields[col.field] = {
                    "type": col.DataType,
                    "editable": _.isUndefined(col.editable) ? false : col.editable
                }
                if (col.DataType === 'number') {
                    //Parse to Number so that localizations(Dutch, Spanish, etc.) would work even for given numbers with "."
                    vm.dataSource.schema.model.fields[col.field].parse = function (num) {
                        if (_.isNull(num) || _.isUndefined(num))
                            return 0;
                        else
                            return Number(num);
                    }
                }
                if (col.DataType === 'date') {
                    /*
                     * We are overriding what the kendo grid does for date columns, at least until we update kendo.
                     * Before this override we had kendo calling kendo.parseDate() internally for columns of type 'date' 
                     * In our current kendo version (2013.3.1119) kendo.parseDate() does not handle UTC dates properly:
                     * 
                     *  post in 2012 saying UTC not supported: 
                     *      http://www.telerik.com/forums/kendo-fails-to-parse-or-format-utc-dates#6uYc1I9XzkCZysvK1g8abg 
                     *  post in 2014 saying fixed: 
                     *      http://www.telerik.com/forums/kendo-fails-to-parse-or-format-utc-dates#mLxRP0gx_Ua_RTlTQBDm3w
                     *  link to fix example: 
                     *      http://dojo.telerik.com/uyaca/3
                     * 
                     * Until we upgrade to the latest kendo version we are intercepting date column parsing and manually 
                     * applying our rules 
                     * 
                     * -David
                     * 
                     */

                    //vm.dataSource.schema.model.fields[col.field].parse = function(dateString) {
                    //    if (_.isNull(dateString) || _.isUndefined(dateString)) {
                    //        return '';
                    //    } else {
                    //        return kendo.toString(new Date(dateString), 'g');
                    //    }
                    //}

                    //Fix for TFS Portal BUG 12131
                    if (col.field.indexOf("AsDatePart") == -1 && vm.grid.GridType == "WorkItem" && vm.grid.dataUrl.toString() != "/grid/GetPromotedView/") {
                        var colField = col.field;
                        var newColumn = col;
                        newColumn.field = col.field + "AsDatePart";
                        vm.grid.columns.push(newColumn);
                        vm.dataSource.schema.model.fields[col.field] = {
                            "type": col.DataType,
                            "editable": _.isUndefined(col.editable) ? false : col.editable
                        }
                    }
                   
                }
            });

            callback();
        }
    }
    return definition;
});
