/* global define: true */

(function() {

	'use strict';

	define(['app'], function(ngApp) { //avoid collisions with app.js in the global space

		ngApp.factory('ChartConfigService', ['$http', '$q', 'LocalizationService', 'notificationService', ChartConfigService]);

		function ChartConfigService($http, $q, localizationService, notificationService) {

			var CONFIG_FILE_BASE_URL = '/Scripts/ng/page/adfWidgets/chart/configFiles/';
			var JSON_FILE_EXT = '.js'; //sigh.  WHY U NO USE .JSON?!?!
			var configMap = {};

			return {
				getConfig: function(configName, groupByColumn, chartType) {
					var deferred = $q.defer();
					var self = this;
					if(configMap[configName]) {
						deferred.resolve(this.initConfig(configMap[configName], groupByColumn, chartType));
					} else {
						var url = this.getConfigUrl(configName);
						$http.get(url)
							.then(function(response) {
								var config = response.data;
								configMap[configName] = config;
								deferred.resolve(self.initConfig(config, groupByColumn, chartType));
							}, function() {
								notificationService.addError("Could not retrieve config file");
							});
					}
					return deferred.promise;
				},
				getConfigUrl: function(configName) {
					var url = CONFIG_FILE_BASE_URL + configName + JSON_FILE_EXT;
					return url;
				},
				getDefaultChartOptions: function() {
			        var chartOptions = {
			        	dataSource: {
			        		data: []
			        	},
			            seriesDefaults: {
			                labels: {
			                    format: "{0}"
			                }
			            },
			            valueAxis: {
			                labels: {
			                    format: "{0}"
			                }
			            }
		        	};
		        	return chartOptions;
				},
				initConfig: function (config, groupByColumn, chartType) {
				    var instance = angular.copy(config);
				    var series = instance.series[0];
                    angular.extend(series, {
                        type: chartType.Id,
                        field: groupByColumn.field,
                        name: groupByColumn.name,
                        categoryField: groupByColumn.field
                    });
                    instance.categoryAxis.baseUnit = groupByColumn.field;

					return instance;
				},
				getChartTypes: function () {
				    var chartTypes = [
                        { Id: 'bar', Name: localizationService.getValue('BarChart') },
                        { Id: 'pie', Name: localizationService.getValue('PieChart') },
                        { Id: 'line', Name: localizationService.getValue('LineChart')},
                        { Id: 'area', Name: localizationService.getValue('AreaChart') },
                        { Id: 'column', Name: localizationService.getValue('ColumnChart') }
				    ];
				    return chartTypes;
				},
				getPieChartSeries: function (config, data) {
				    //"At present assigning aggregate fields to the chart series is not supported. 
				    //As a workaround, create the series array manually and pass it to the chart configuration." 
				    //-taken from this forum: http://www.telerik.com/forums/aggregate-pie-chart and this example: http://jsfiddle.net/valchev/VzWdk/

				    var seriesData = [];
				    var dataSource = new kendo.data.DataSource({
				        data: data,
				        group: {
				            field: config.groupByColumn.field,
				            aggregates: [
                                { field: "Index", aggregate: "count" }
				            ]
				        }
				    });

				    dataSource.read();
				    angular.forEach(dataSource.view(), function (item) {
				        seriesData.push({ category: item.value, value: item.aggregates.Index.count });
				    });

				    var series = [{
				        data: seriesData,
				        type: config.chartType.Id,
				        categoryField: config.groupByColumn.field
				    }];

				    return series;
				}
			};
		}

	});

})();