/* global define: true */

(function() {

	'use strict';

	define(['app'], function(ngApp) { 

		ngApp.factory('ODataChartConfigService', ['$http', '$q', 'LocalizationService', 'notificationService', ODataChartConfigService]);

		function ODataChartConfigService($http, $q, localizationService, notificationService) {

			var CONFIG_FILE_BASE_URL = '/Scripts/ng/page/adfWidgets/odataChart/configFiles/';
			var JSON_FILE_EXT = '.js'; 
			var configMap = {};
			var colorIndex = 0;
			return {
			    getConfig: function (configName, groupByColumn, chartType) {
					var deferred = $q.defer();
					var self = this;
					if(configMap[configName]) {
						deferred.resolve(this.initConfig(configMap[configName], groupByColumn, chartType));
					} else {
						var url = this.getConfigUrl(configName);
						$http.get(url)
							.then(function(response) {
							    var config = response.data;
								configMap[configName] = config;
								deferred.resolve(self.initConfig(config, groupByColumn, chartType));
							}, function() {
								notificationService.addError("Could not retrieve config file");
							});
					}
					return deferred.promise;
			    },
			    rewriteFilterParams: function (url) {
			        var queryString = url.substr(url.indexOf('?'), url.length);
			        url = url.substr(0, url.indexOf('?') + 1);
			        var params = this.getQueryParams(queryString)._wrapped || [];
			        var rewriteURL = url;
			        var filterCount = 0;
			        var filterStr = "$filter=";
			        var otherFiltersStr = "";
			        var topParam = _.find(params, function (item) {
			            return item[0] == "$top";
			        });
			        var bTopParamChange = false;

			        //set max record count to 500 if top is not defined. used to fix issue with exportToExcel function
			        if (_.isUndefined(topParam)) {
			            params.push({ 0: "$top", 1: "500" });
			            bTopParamChange = true;
			        }

			        for (var i in params) {
			            if (params[i][0] == '$top') {
			                //set max record count to 500. used to fix issue with exportToExcel function
			                if (params[i][1] > 500) {
			                    params[i][1] = 500;
			                    bTopParamChange = true;
			                }
			            }

			            if (params[i][0] == '$filter') {
			                filterStr += params[i][1] + ' and '
			                filterCount++;
			            } else {
			                if (params[i][0].length > 0 && params[i][0] != '$format') {
			                    params[i][1] = (params[i][1]) ? params[i][1] : '';
			                    otherFiltersStr += params[i][0] + "=" + params[i][1] + "&";
			                }
			            }
			        }

			        if (filterStr == "$filter=") return url;

                    var checkEnd = filterStr.substr(filterStr.length - 5, filterStr.length);
                    if (checkEnd == ' and ')
                        filterStr = filterStr.substr(0, filterStr.length - 5);

			        if (filterCount > 1) {
			            if (otherFiltersStr.length > 0)
			                rewriteURL += filterStr + '&' + otherFiltersStr;
			            else
			                rewriteURL += filterStr;

			            return rewriteURL;
			        } else {
			            if (bTopParamChange) {
			                if (otherFiltersStr.length > 0)
			                    rewriteURL += filterStr + '&' + otherFiltersStr;
			                else
			                    rewriteURL += filterStr;
			                return rewriteURL;
			            }
			        }

			        return false;
			    },
			    getQueryParams: function (queryString) {
			        var query = (queryString || window.location.search).substring(1); // delete ?
			        if (!query) {
			            return false;
			        }
			        return _
                    .chain(query.split('&'))
                    .map(function (params) {
                        var p = params.split('=');
                        return [decodeURIComponent(p[0].toLowerCase()), p[1]];
                    })
			    },
				getConfigUrl: function(configName) {
					var url = CONFIG_FILE_BASE_URL + configName + JSON_FILE_EXT;
					return url;
				},
				getDefaultChartOptions: function() {
			        var chartOptions = {
			        	dataSource: {
			        		data: []
			        	},
			            seriesDefaults: {
			                labels: {
			                    format: "{0}"
			                }
			            },
			            valueAxis: {
			                labels: {
			                    format: "{0}"
			                }
			            }
		        	};
		        	return chartOptions;
				},
				initConfig: function (config, groupByColumn, chartType) {
				    var instance = angular.copy(config);
				    var series = instance.series[0];
                    angular.extend(series, {
                        type: chartType.Id,
                        field: groupByColumn.field,
                        name: groupByColumn.name,
                        categoryField: groupByColumn.field
                    });
                    instance.categoryAxis.baseUnit = groupByColumn.field;

					return instance;
				},
				getChartTypes: function () {
				    var chartTypes = [
                        { Id: 'bar', Name: localizationService.getValue('BarChart') },
                        { Id: 'pie', Name: localizationService.getValue('PieChart') },
                        { Id: 'line', Name: localizationService.getValue('LineChart')},
                        { Id: 'area', Name: localizationService.getValue('AreaChart') },
                        { Id: 'column', Name: localizationService.getValue('ColumnChart') },
                        { Id: 'donut', Name: localizationService.getValue('DonutChart') },
                        { Id: 'stackedarea', Name: localizationService.getValue('StackedAreaChart') },
                        { Id: 'stackedbar', Name: localizationService.getValue('StackedBarChart') },
                        { Id: 'stackedcolumn', Name: localizationService.getValue('StackedColumnChart') }
				    ];
				    return chartTypes;
				},
				getCustomChartOptions: function (dataSource, config, options) {

				        var seriesData = [];
				        var categories = [];
				        var colors = options.seriesColors;
				        angular.forEach(dataSource.view(), function (item) {
				            var category = (config.applyGrouping) ? item.value : item[config.label.field];
				            var value = (config.applyGrouping) ? item.aggregates.Index.count : item[config.groupByColumn.field];
				            seriesData.push({ category: category, value: value, color: colors[colorIndex++ % colors.length] });
				            categories.push(category);
				        });

				        var series = [{
				            data: seriesData,
				            type: config.chartType.Id
				        }];

				        options.categoryAxis.categories = categories;
				        options.series = series;

				        options.tooltip = {
				            visible: true,
				            template: "#= dataItem.category # - #= value #"
				        };

				        return options;
				},
				setErrorNotification: function (message) {
				    notificationService.addError(message);
				}
			};
		}

	});

})();