/* global define: true */
/* global _: true */

(function () {
    'use strict';

    define([
		'app',
    ], function (/*app*/) {

        function ODataCountController($sce, $scope, config, localizationService, gridConfigService) {
            $scope.config.count = config.count || 0;
            var vm = this;
            var doURL = config.data_url;

            if (doURL) {

                var conString = config.queryString || '';
                var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';

                if (conString.length > 0) {

                    var urlParams = app.lib.getQueryParams() || {};

                    _.each(config.queryParamKeys,
                        function (item) {
                            var itemKey = item.Key.toLowerCase();
                            if (urlParams.hasOwnProperty(itemKey))
                                conString = conString.replace("{{" + item.Key + "}}", urlParams[itemKey]);
                            else
                                conString = conString.replace("{{" + item.Key + "}}", item.Value);
                        });

                    if (conString.indexOf('@User') != -1) {
                        conString = conString.replace('@User', session.user.Id)
                    }

                    if (conString.indexOf('@Lang') != -1) {
                        conString = conString.replace('@Lang', lang)
                    }
                }

                var invalids = ["$top", "$count", "$format", "$sort"];

                for (var i in invalids) {
                    if (conString.indexOf(invalids[i]) > -1) {
                        kendo.ui.ExtAlertDialog.show({
                            title: localization.Warning,
                            message: localization.ODATAQueryInvalid
                        });
                        return;
                    }
                }

                var langCodeField = {};

                var el = document.createElement('a');
                el.href = config.data_url;
                var path = el.pathname;
                vm.entity = path.substring(path.lastIndexOf("/") + 1, path.length);

                gridConfigService.getColumnConfig(vm.entity).then(function(res) {

                    langCodeField = res["LanguageCode"];

                    if (langCodeField.field &&
                        conString.indexOf("LanguageCode") == -1) {
                        var urlParams = app.lib.getQueryParams(conString) || {};
                        var filterItems = urlParams["$filter"] || "";
                        var filterStr = (filterItems.length > 0)
                            ? filterItems + "and LanguageCode eq '" + lang + "'"
                            : filterItems + "LanguageCode eq '" + lang + "'";
                        urlParams["$filter"] = filterStr;
                        
                        conString = "";
                        for (var key in urlParams) {
                            if (urlParams.hasOwnProperty(key)) {
                                conString += (conString.length > 0)
                                    ? "&" + key + "=" + urlParams[key]
                                    : "?" + key + "=" + urlParams[key];
                            }
                        }
                    }

                    doURL += (conString.length > 0)
                        ? conString + "&$format=json&$top=0&$count=true"
                        : "?$format=json&$top=0&$count=true";

                    $.getJSON(doURL,
                        function(result) {
                            if (result['@odata.count'])
                                $scope.config.count = result['@odata.count'];
                            else
                                $scope.config.count = result.value.length;

                            $scope.$apply();

                        }).fail(function(err) {
                        console.log(err);
                    });
                });
            }
        }

        function ODataCountEditController($sce, $scope, config, localizationService, notificationService) {

            $scope.config = angular.extend(config, {
                given_url: config.given_url || null,
                data_url: config.data_url || null,
                queryString: config.queryString || '',
                gridCollection: config.gridCollection || [],
                queryParamKeys: config.queryParamKeys || []
            });

            var queryParamKeys = _.uniq(config.queryString.match(/{{([^}]+)}}/g));
            //Add new parameters
            _.each(queryParamKeys,
                function (key) {

                    var token = key.replace('{{', '').replace('}}', '');

                    var exist = _.filter(config.queryParamKeys,
                        function (item) {
                            return item.Key === token;
                        });

                    if (exist.length === 0) {
                        config.queryParamKeys.push({ Key: key.replace('{{', '').replace('}}', ''), Value: '' });
                    }
                });

            var conString = config.queryString || '';
            var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';

            if (conString.length > 0) {

                _.each(config.queryParamKeys,
                    function (item) {
                        conString = conString.replace("{{" + item.Key + "}}", item.Value);
                    });

                if (conString.indexOf('@User') != -1) {
                    conString = conString.replace('@User', session.user.Id)
                }

                if (conString.indexOf('@Lang') != -1) {
                    conString = conString.replace('@Lang', lang)
                }
            }

            var vm = this;
            angular.extend(vm, {
                localize: function (key) {
                    return localizationService.getValue(key);
                },
                collections: [$scope.config.gridCollection] || [],
                queryInvalid: function() {
                    var invalids = ["$top", "$count", "$format", "$sort"];

                    for (var i in invalids) {
                        if (config.queryString.indexOf(invalids[i]) > -1) {
                            $('#saveAdfDialog').attr("disabled", "disabled");
                            return true;
                        }
                    }
                    $('#saveAdfDialog').removeAttr("disabled");
                    return false;
                },
                getCollections: function (init) {

                    $('#collections').attr('disabled', true);

                    var url = config.given_url
                    var doURL = config.given_url;

                    doURL += "?$format=json&$top=0&$count=true";

                    if (doURL) {
                        $.getJSON(doURL,
                        function (result) {
                            var data = result.value;
                            var sets = data || [];
                            var kind = (data[0]) ? data[0].kind : '';
                        
                            if (kind == 'EntitySet') {
                                $('#collections').removeAttr("disabled");

                                for (var i in sets) {
                                    if (sets[i].name) {
                                        vm.collections.push({
                                            field: sets[i].name,
                                            title: sets[i].name
                                        });
                                    }
                                }

                                if ($scope.config.gridCollection.length != 0)
                                    url += '/' + $scope.config.gridCollection.field;
                            } else
                                $scope.config.gridCollection = [];

                            if (!init) {
                                if (result['@odata.count'])
                                    config.count = result['@odata.count'];
                                else
                                    config.count = result.value.length;
                            }

                            $scope.config.data_url = url;

                        }).fail(function (err) {
                            $scope.config.gridCollection = [];
                            console.log(err);
                        });

                    }
                },
                changeCollection: function (url) {

                    config.data_url = url;

                    if (url) {

                        var doURL = config.data_url;

                        doURL += (conString.length > 0) ? conString + "&$format=json&$top=1&$count=true" : "?$format=json&$top=1&$count=true";

                        $.getJSON(doURL,
                        function (result) {
                            if (result['@odata.count'])
                                config.count = result['@odata.count'];
                            else
                                config.count = result.value.length;
                        }).fail(function (err) {
                            console.log(err);
                        });
                    }
                },
                onChangeQueryString: function () {
                    var queryParamKeys = _.uniq(config.queryString.match(/{{([^}]+)}}/g));

                    //Add new parameters
                    _.each(queryParamKeys,
                        function (key) {

                            var token = key.replace('{{', '').replace('}}', '');

                            var exist = _.filter(config.queryParamKeys,
                                function (item) {
                                    return item.Key === token;
                                });

                            if (exist.length === 0) {
                                config.queryParamKeys.push({ Key: key.replace('{{', '').replace('}}', ''), Value: '' });
                            }
                        });

                    //Delete unused parameters
                    var i = 0;
                    _.each(config.queryParamKeys,
                        function (item) {

                            var exist = _.filter(queryParamKeys,
                                function (key) {
                                    var token = key.replace('{{', '').replace('}}', '');
                                    return item.Key === token;
                                });

                            if (exist.length === 0)
                                config.queryParamKeys.splice(i, 1);

                            i++;
                        });
                }
            });

            vm.getCollections(true);
        }

        angular.module('adf.widget.odata.count', ['adf.provider'])
          .config(["dashboardProvider", function (dashboardProvider) {
              dashboardProvider
                .widget('odata-count', {
                    title: localizationHelper.localize('AdfODataCountWidgetTitle', 'OData Count Widget'), //localization key
                    description: localizationHelper.localize('AdfODataCountWidgetDescription', 'Add a count widget based on an OData End Point'), //localization key
                    templateUrl: '{widgetsPath}/odata-count/src/view.html',
                    controller: 'ODataCountController',
                    controllerAs: 'vm',
                    edit: {
                        templateUrl: '{widgetsPath}/odata-count/src/edit.html',
                        controller: 'ODataCountEditController',
                        controllerAs: 'vm'
                    },
                    reload: true,
                    ordinal: 9
                });
          }])
          .controller('ODataCountController', ['$sce', '$scope', 'config', 'LocalizationService', 'ODataGridConfigService', ODataCountController])
          .controller('ODataCountEditController', ['$sce', '$scope', 'config', 'LocalizationService', 'notificationService', ODataCountEditController]);

        var editViewTemplate = ["<form role=form><div class=form-group>",
                "<label for=url>{{ ::vm.localize('ODataEndPoint') }}</label>",
                "<input type=text id=url ng-model=\"config.given_url\" class=\"form-control input-sm\" ng-change=\"vm.getCollections()\" />",
                "</div><div class=form-group>",
                "<label for=collections>{{ ::vm.localize('ChooseCollection') }}</label>",
                "<select id=collections class=\"form-control input-sm\" ng-model=\"config.gridCollection\" ng-change=\"vm.changeCollection(config.given_url + '/' + config.gridCollection.field)\" ng-options=\"c.title for c in vm.collections track by c.field\">",
                "</select>",
                "</div><div class=form-group>",
                "<label for=queryString>{{ ::vm.localize('QueryString') }}</label>",
                "<input type=text id=queryString ng-model=\"config.queryString\" class=\"form-control input-sm\" ng-change=\"vm.onChangeQueryString()\"/>",
                "<p class='help-block' style='color: red' ng-if=\"vm.queryInvalid()\">{{ ::vm.localize('ODATAQueryInvalid') }}</p>",
                "</div>",
                "<div class=\"form-horizontal\">",
                    "<div class=form-group ng-if=\"config.queryParamKeys.length>0\">",
                        "<ul id=\"param-content\"><li ng-repeat=\"param in config.queryParamKeys track by $index\">",
                            "<div class=\"query-param\">",
                                "<label class=\"col-sm-3 control-label\">{{param.Key}}</label>",
                                "<div class=\"col-sm-9\"><input type=\"text\" class=\"k-textbox form-control input-sm \" ng-model=\"param.Value\"/></div>",
                            "</div>",
                        "</li></ul>",
                    "</div>",
                "</div>",
                "</form>"].join("");

        var mainViewTemplate = [
            "<div class=\"count-widget-container\">",
                 "<div class=\"content\">{{config.count}}</div>",
             "</div>"
        ].join('');

        angular.module("adf.widget.odata.count").run(["$templateCache", function ($templateCache) {
            $templateCache.put("{widgetsPath}/odata-count/src/edit.html", editViewTemplate);
            $templateCache.put("{widgetsPath}/odata-count/src/view.html", mainViewTemplate);
        }]);
    });
})();