﻿/* global define: true */
/* global _: true */

(function () {

    'use strict';

    define([
        'app'
    ], function ( /*app*/) {

        function ODataService($http, $q, navigationNodeService, viewPanelService) {
            return {
                getData: function (url) {
                    var deferred = $q.defer();
                    url += "?$format=json&$top=1";

                    $.getJSON(url,
                    function (result) {
                        deferred.resolve(result.value);
                    }).fail(function (err) {
                        deferred.reject(err)
                    });

                    return deferred.promise;
                }
            };
        }

        function ODtataGridEditController($scope, config, ODataService, ODataGridConfigService, localizationService) {

            $scope.config = angular.extend(config, {
                isCustomMobile: config.isCustomMobile || false,
                mobGridGUID: config.mobGridGUID || ODataGridConfigService.getGuid(),
                customTemplate: config.customTemplate || '',
                given_url: config.given_url || null,
                data_url: config.data_url || null,
                queryString: config.queryString || '',
                gridColumns: config.gridColumns || [],
                allColumns: config.allColumns || [],
                gridCollection: config.gridCollection || [],
                configurationName: "odataGridConfig",
                queryParamKeys: config.queryParamKeys || []
            });

            $scope.config.gridColumns = _.filter($scope.config.gridColumns,
                function (el) {
                    return !el.hidden;
                });

            var queryParamKeys = _.uniq(config.queryString.match(/{{([^}]+)}}/g));
            //Add new parameters
            _.each(queryParamKeys,
                function (key) {

                    var token = key.replace('{{', '').replace('}}', '');

                    var exist = _.filter(config.queryParamKeys,
                        function (item) {
                            return item.Key === token;
                        });

                    if (exist.length === 0) {
                        config.queryParamKeys.push({ Key: key.replace('{{', '').replace('}}', ''), Value: '' });
                    }
                });

            var vm = this;
            angular.extend(vm, {
                entity: '',
                queryInvalid: function() {
                    var invalids = ["$top", "$count", "$format", "$sort"];

                    for (var i in invalids) {
                        if (config.queryString.indexOf(invalids[i]) > -1) {
                            $('#saveAdfDialog').attr("disabled", "disabled");
                            return true;
                        }
                    }
                    $('#saveAdfDialog').removeAttr("disabled");
                    return false;
                },
                isCustomMobile: $scope.config.isCustomMobile,
                isCustomGrid: $scope.config.isCustomGrid,
                useCustomGrid: function () {

                    if (!$("#" + vm.entity + "_tpl").html()) {

                        var initTemplate = '<tr data-uid="#: uid #"> \n';
                        if (vm.isCustomGrid && config.gridColumns.length > 0) {
                            angular.forEach(config.gridColumns, function (item, index) {
                                initTemplate += '<td>' + item.template + '</td> \n'
                            });
                        }

                        initTemplate += '</tr>'

                        config.customGridTemplate = initTemplate;
                    } else {
                        config.customGridTemplate = $("#" + vm.entity + "_tpl").html()
                    }
                },
                useCustomMobile: function () {

                    var initTemplate = '<div class="gridcard gridcard--asset"> \n';

                    if (!$("#" + vm.entity + "_tpl").html()) {

                        if (vm.isCustomMobile && config.gridColumns.length > 0) {

                            angular.forEach(config.gridColumns, function (item, index) {
                                if (index == 0) {
                                    initTemplate += '<p class="gridcard__title"> \n';
                                    initTemplate += '<a href="">#: ' + item.field + ' #</a> \n </p> \n';
                                    initTemplate += '<div class="gridcard__detailblock"> \n';
                                } if (index == config.gridColumns.length - 1) {
                                    initTemplate += '<p>' + item.template + '</p> \n </div> \n'
                                } else {
                                    initTemplate += '<p>' + item.template + '</p> \n'
                                }
                            });
                        }
                        initTemplate += '</div>'

                        config.customTemplate = initTemplate;
                    } else {
                        config.customTemplate = $("#" + vm.entity + "_mtpl").html()
                    }
                },
                localize: function (key) {
                    return localizationService.getValue(key);
                },
                gridColumns: _.map(config.gridColumns, function (el) {
                    return el.title;
                }),
                collections: [$scope.config.gridCollection] || [],
                getCollections: function (url) {

                    $('#columns').attr('disabled', true);
                    $('#collections').attr('disabled', true);

                    if ($("#columns").parent().hasClass("k-multiselect")) {
                        vm.gridColumns = [];
                        $("#columns").parent().replaceWith("<select id=columns disabled class=\"form-control input-sm\" ng-model=\"gridColumns\">")
                    }
                    
                    if (!url)
                        url = config.given_url;

                    if (url) {
                        ODataService.getData(config.given_url).then(function (data) {

                            var sets = data || [];
                            if (data[0].kind == 'EntitySet') {
                                $('#collections').removeAttr("disabled");
                                for (var i in sets) {
                                    if (sets[i].name) {
                                        vm.collections.push({
                                            field: sets[i].name,
                                            title: sets[i].name
                                        });
                                    }
                                }

                                if ($scope.config.gridCollection.length != 0) {
                                    url += '/' + $scope.config.gridCollection.field;
                                    vm.getColumns(url);
                                }
                            } else {
                                $scope.config.gridCollection = [];
                                $scope.config.data_url = url;

                                vm.setColumns(data);
                            }
                        });
                    }
                },
                getColumns: function (url) {

                    $scope.config.data_url = url;

                    ODataService.getData(url).then(function (data) {
                        vm.setColumns(data);
                    });
                },
                setColumns: function (data) {

                    var el = document.createElement('a');
                    el.href = config.data_url;
                    var path = el.pathname;
                    vm.entity = path.substring(path.lastIndexOf("/") + 1, path.length);
                   
                    $('#columns').removeAttr("disabled");

                    var dbColumns = config.allColumns;

                    if (data.length > 0 && dbColumns.length == 0) {

                        if ($("#" + vm.entity + "_tpl").html()) {
                            ODataGridConfigService.getColumnConfig(vm.entity).then(function (res) {
                                vm.columnTemplate(data, res);
                            });
                        }else
                            vm.columnTemplate(data, []);
                    } else {
                        vm.configColumnMultiSelect(dbColumns);
                    }
                },
                columnTemplate: function (data, defaultColumns) {

                    var dbColumns = [];
                    angular.forEach(_.keys(data[0]), function (item, index) {
                        if (item != '__metadata') {

                            var templ = "#= (" + item + ") ? " + item + " : '' #";
                            var dataType = '';

                            var val = data[0][item];

                            if (!val) {
                                for (var i = 1; i < data.length; i++) {
                                    if (data[i][item]) {
                                        val = data[i][item];
                                        i = data.length
                                    }
                                }
                            }

                            //check if value is null, if not then convert to string to prevent  error in replace and regex methods
                            if (val) {
                                val = val.toString();

                                //trim white spaces to prevent parsing strings ending with a number as date
                                val = val.replace(/\s+/, "");

                                var guidCheck = new RegExp("^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$");

                                if (Date.parse(val) && /^\d+$/.test(val) == false && parseFloat(val)) {
                                    //prevent parsing number objects and link strings as date 
                                    templ = "#= (" + item + ") ? kendo.toString(new Date(" + item + "), 'M/d/yyyy h:mm tt') : '' #";
                                    dataType = 'date';
                                } else if (parseFloat(val) && !isNaN(val)) {
                                    templ = "#= (" + item + ") ? " + kendo.toString(item, 'n') + " : 0 #";
                                    dataType = 'number';
                                } else if (guidCheck.test(val)) {
                                    dataType = 'GUID';
                                } else {
                                    dataType = 'string';
                                }
                            }

                            dbColumns.push({
                                field: (defaultColumns[item]) ? defaultColumns[item].field : item,
                                title: (defaultColumns[item]) ? localizationHelper.localize(defaultColumns[item].title, defaultColumns[item].title) : localizationHelper.localize(item, item),
                                template: (defaultColumns[item]) ? defaultColumns[item].template : templ,
                                DataType: (defaultColumns[item]) ? defaultColumns[item].DataType : dataType,
                                hidden: (defaultColumns[item]) ? defaultColumns[item].hidden : false
                            });
                        }
                    });

                    vm.configColumnMultiSelect(dbColumns);
                },
                configColumnMultiSelect: function (dbColumns) {

                    if (dbColumns.length > 0) $("#columns").removeClass("form-control input-sm");

                    config.allColumns = dbColumns;

                    vm.selectOptions = {
                        placeholder: vm.localize("ChooseColumns"),
                        dataTextField: "title",
                        dataValueField: "title",
                        valuePrimitive: false,
                        autoBind: true,
                        change: function () {

                            var dataItems = $('#columns').data('kendoMultiSelect').dataItems() || [];

                            if (vm.gridColumns.length > 0) {
                                for (var i in vm.gridColumns) {
                                    if (vm.gridColumns[i].tagName == "LI") {
                                        var text = vm.gridColumns[i].innerText;
                                        text = text.substring(0, text.length - 6).toString().toLowerCase();

                                        for (var x in config.gridColumns) {
                                            if (text == config.gridColumns[x].title.toString().toLowerCase()) {
                                                dataItems.push(config.gridColumns[x]);
                                            }
                                        }
                                    }
                                }
                            }

                            config.gridColumns = dataItems;

                            if(dataItems.length > 0)
                                $('.k-multiselect-wrap.k-floatwrap').addClass('columnsselected');
                            else
                                $('.k-multiselect-wrap.k-floatwrap').removeClass('columnsselected');

                            $("#columns_taglist > li").css("cssText", "cursor: pointer; height:inherit !important;");
                            $("#columns_taglist > li > .k-select").css("cursor", "pointer");
                        },
                        dataSource: new kendo.data.DataSource({
                            data: config.allColumns
                        }),
                        value: vm.gridColumns
                    };

                    var cols = false;

                    if ($("#columns").parent().hasClass("k-multiselect")) {
                        cols = $("#columns").data("kendoMultiSelect").setDataSource(new kendo.data.DataSource({ data: dbColumns }));
                        $("#columns_taglist > li").css("cssText", "cursor: move; height:inherit !important;");
                        $("#columns_taglist > li > .k-select").css("cursor", "pointer");
                    } else {
                        cols = $("#columns").kendoMultiSelect(vm.selectOptions).data("kendoMultiSelect");
                    }

                    if (config.gridColumns.length > 0)
                        $('.k-multiselect-wrap.k-floatwrap').addClass('columnsselected');
                    else
                        $('.k-multiselect-wrap.k-floatwrap').removeClass('columnsselected');

                    if (cols) {

                        $("#columns_taglist > li").css("cssText", "cursor: pointer; height:inherit !important;");
                        $("#columns_taglist > li > .k-select").css("cursor", "pointer");

                        cols.tagList.kendoSortable({
                            hint: function (element) {
                                return element.clone().addClass("hint");
                            },
                            placeholder: function (element) {
                                return element.clone().addClass("placeholder").text("drop here");
                            },
                            change: function (e) {

                                var listElements = e.sender.element.children();

                                var listValues = [];

                                if (vm.gridColumns.length > 0) {

                                    var dataItems = [];

                                    for (var a in config.gridColumns) {
                                        var found = false;
                                        for (var c in vm.gridColumns) {
                                            var text1 = vm.gridColumns[c].toString().toLowerCase();
                                            if (text1 === config.gridColumns[a].title.toString().toLowerCase()) {
                                                dataItems.push(config.gridColumns[a]);
                                                found = true;
                                            }
                                        }
                                        if (!found) dataItems.push(config.gridColumns[a]);
                                    }

                                    config.gridColumns = dataItems;
                                }

                                for (var i in listElements) {
                                    if (listElements[i].tagName == "LI") {
                                        var text = listElements[i].innerText.toLowerCase();
                                        for (var x in config.gridColumns) {
                                            if (text === config.gridColumns[x].title.toString().toLowerCase()) {
                                                listValues.push(config.gridColumns[x]);
                                            }
                                        }
                                    }
                                }

                                config.gridColumns = listValues;
                            }
                        });
                    }

                    if ($("#columns").data("kendoMultiSelect")) {
                        if (config.queryString.indexOf("$select") !== -1)
                            $("#columns").data("kendoMultiSelect").enable(false);
                        else
                            $("#columns").data("kendoMultiSelect").enable(true);
                    }
                },
                onChangeQueryString: function() {
                    var queryParamKeys = _.uniq(config.queryString.match(/{{([^}]+)}}/g));

                    //Add new parameters
                    _.each(queryParamKeys,
                        function (key) {

                        var token = key.replace('{{', '').replace('}}', '');

                        var exist = _.filter(config.queryParamKeys,
                            function (item) {
                                return item.Key === token;
                            });

                        if (exist.length === 0) {
                            config.queryParamKeys.push({ Key: key.replace('{{', '').replace('}}', ''), Value: '' });
                        } 
                        });

                    //Delete unused parameters
                    var i = 0;
                    _.each(config.queryParamKeys,
                        function (item) {

                            var exist = _.filter(queryParamKeys,
                                function (key) {
                                    var token = key.replace('{{', '').replace('}}', '');
                                    return item.Key === token;
                                });

                            if (exist.length === 0)
                                config.queryParamKeys.splice(i, 1);

                            i++;
                        });

                    if (config.queryString.indexOf("$select") !== -1)
                        $("#columns").data("kendoMultiSelect").enable(false);
                    else
                        $("#columns").data("kendoMultiSelect").enable(true);
                }
            });

            vm.getCollections();
        }

        function ODataGridController($scope, config, ODataGridConfigService, localizationService) {
            $scope.stateGuid = config.stateGuid;

            var newQueryString = config.queryString || '';
            var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';
            var urlParams = app.lib.getQueryParams() || {};

            if (newQueryString.length > 0) {

                _.each(config.queryParamKeys,
                    function (item) {
                        var itemKey = item.Key.toLowerCase();
                        if (urlParams.hasOwnProperty(itemKey))
                            newQueryString = newQueryString.replace("{{" + item.Key + "}}", urlParams[itemKey]);
                        else
                            newQueryString = newQueryString.replace("{{" + item.Key + "}}", item.Value);
                    });

                if (newQueryString.indexOf('@User') !== -1) {
                    newQueryString = newQueryString.replace('@User', session.user.Id);
                }

                if (newQueryString.indexOf('@Lang') !== -1) {
                    newQueryString = newQueryString.replace('@Lang', lang);
                }
            }

            var invalids = ["$top", "$count", "$format", "$sort"];

            for (var i in invalids) {
                if (newQueryString.indexOf(invalids[i]) > -1) {
                    kendo.ui.ExtAlertDialog.show({
                        title: localization.Warning,
                        message: localization.ODATAQueryInvalid
                    });
                    return;
                }
            }

            $scope.$on("kendoWidgetCreated", function (event, widget) {

                widget.dataSource.transport.parameterMap = function (options, type) {

                    var paramMap = kendo.data.transports["odata-v4"].parameterMap(options);

                    var filters = options.filter || [];
                    for (var i in filters) {

                        if (filters[i].constructor === Array) {
                            var innerFilters = filters[i];
                            for (var x in innerFilters) {
                                if (innerFilters[x].filters) {
                                    var nestedFilters = innerFilters[x].filters;
                                    for (var q in nestedFilters) {
                                        var val = nestedFilters[q].value.toString().replace(/\s+/, "");
                                        if (parseFloat(val)) {
                                            var n = val.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                                            var operator = nestedFilters[q].operator;
                                            operator = (operator == "lte") ? "le" : (operator == "gte") ? "ge" : operator;
                                            var fil = nestedFilters[q].field + " " + operator + " ";
                                            if (paramMap.$filter.indexOf(fil + n) != -1) {
                                                paramMap.$filter = paramMap.$filter.replace(fil + n, fil + nestedFilters[q].value)
                                            }
                                        }
                                    }
                                } else {
                                    var val = innerFilters[x].value.toString().replace(/\s+/, "");
                                    if (parseFloat(val)) {
                                        var n = val.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
                                        var operator = innerFilters[x].operator;
                                        operator = (operator == "lte") ? "le" : (operator == "gte") ? "ge" : operator;
                                        var fil = innerFilters[x].field + " " + operator + " ";
                                        if (paramMap.$filter.indexOf(fil + n) != -1) {
                                            paramMap.$filter = paramMap.$filter.replace(fil + n, fil + innerFilters[x].value)
                                        }
                                    }
                                }
                            }
                        }
                       
                    }
                    paramMap.$count = true;
                    delete paramMap.$inlinecount; // <-- remove inlinecount parameter.
                    return paramMap;
                }

                if (widget === event.currentScope.grid) {

                    var gridId = event.currentScope.stateGuid;
                    var currentState = app.gridUtils.savedState.getCurrentState(gridId);
                    if (currentState) {
                        widget.dataSource.group(currentState.group);
                        if (!_.isUndefined(widget.dataSource.pageSize())) {
                            widget.dataSource.pageSize(currentState.pageSize);
                        }
                        if (!_.isUndefined(widget.dataSource.page())) {
                            widget.dataSource.page(currentState.page);
                        }
                        widget.dataSource.sort(currentState.sort);

                        var dsFilters = currentState.filter || [];
                        if (!(dsFilters instanceof Array))
                            dsFilters = dsFilters.filters;

                        dsFilters = _.filter(dsFilters, function (el) {
                            return el.name != 'searchFilters';
                        });
                        dsFilters = _.map(dsFilters, function (el) {

                            if(el.value){
                                var val = el.value.toString();
                                //trim white spaces to prevent parsing strings ending with a number as date
                                val = val.replace(/\s+/, "");
                                if (Date.parse(val) && /^\d+$/.test(val) == false && parseFloat(val))
                                    el.value = new Date(el.value);
                            }

                            return el;
                        });

                        var langCodeField = _.filter(config.allColumns, function (el) {
                            return el.field == "LanguageCode";
                        });

                        var langCodeFieldFilters = _.filter(dsFilters, function (el) {
                            return el.name == 'sysFilters';
                        });

                        var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';

                        if (langCodeField.length > 0 && langCodeFieldFilters.length == 0 && newQueryString.indexOf("LanguageCode") == -1) {
                            dsFilters = _.filter(dsFilters, function (el) {
                                return el.name != 'sysFilters';
                            });
                            dsFilters.push({
                                logic: 'or',
                                filters: [{
                                    field: "LanguageCode",
                                    operator: 'eq',
                                    value: lang
                                }],
                                name: 'sysFilters'
                            });
                        } else if (langCodeField.length > 0 && langCodeFieldFilters.length > 1 || newQueryString.indexOf("LanguageCode") > -1) {
                            dsFilters = _.filter(dsFilters, function (el) {
                                return el.name != 'sysFilters';
                            });
                        }

                        widget.dataSource.filter(dsFilters);
                    } else {
                        var dsFilters = widget.dataSource.filter() || [];

                        var langCodeField = _.filter(config.allColumns, function (el) {
                            return el.field == "LanguageCode";
                        });

                        var langCodeFieldFilters = _.filter(dsFilters, function (el) {
                            return el.name == 'sysFilters';
                        });

                        var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';

                        if (langCodeField.length > 0 && langCodeFieldFilters.length == 0 && newQueryString.indexOf("LanguageCode") == -1) {
                            dsFilters = _.filter(dsFilters, function (el) {
                                return el.name != 'sysFilters';
                            });
                            dsFilters.push({
                                logic: 'or',
                                filters: [{
                                    field: "LanguageCode",
                                    operator: 'eq',
                                    value: lang
                                }],
                                name: 'sysFilters'
                            });
                        } else if (langCodeField.length > 0 && langCodeFieldFilters.length > 1 || newQueryString.indexOf("LanguageCode") > -1) {
                            dsFilters = _.filter(dsFilters, function (el) {
                                return el.name != 'sysFilters';
                            });
                        }

                        widget.dataSource.filter(dsFilters);
                    }

                    $('.clear-filters-' + gridId).on('click', function () {
                        app.gridUtils.savedState.removeSavedState(gridId);
                        document.location.reload(false);
                    });
                }
            });

            var el = document.createElement('a');
            el.href = config.data_url;
            var path = el.pathname;
            var entity = path.substring(path.lastIndexOf("/") + 1, path.length);
            var vm = this;
            config.gridColumns = config.gridColumns || [];
            config.allColumns = config.allColumns || [];
            angular.extend(vm, {
                gridOptions: null,
                gridColumns: (config.gridColumns.length > 0) ? config.gridColumns : config.allColumns || [],
                onReorderColumn: function (e) {

                    setTimeout(function () {
                        //Do something for reordering columns
                    }, 0);
                },
                getGridOptions: function () {
                    var configName = config.configurationName || "odataGridConfig";
                    vm.gridStateId = config.stateGuid;

                    var columns = vm.gridColumns || [];
                    var selectQuery = "";

                    var guidColumn = _.filter(columns,
                            function (el) {
                                return el.field === "Guid";
                            });

                    var guidColumnAll = _.filter(config.allColumns,
                        function (el) {
                            return el.field === "Guid";
                        });
                    if (guidColumn.length === 0 && guidColumnAll.length > 0) {
                        guidColumnAll[0].hidden = true;
                        columns.push(guidColumnAll[0]);
                    }

                    if (newQueryString.indexOf("$select") === -1) {
                        if (columns.length !== config.allColumns.length) {
                            var selection1 = _.map(columns,
                                function (el) {
                                    return el.field;
                                });

                            selectQuery = "$select=" + selection1.toString();
                            selectQuery = (newQueryString.length > 0) ? "&" + selectQuery : "?" + selectQuery;
                        }
                    } else {
                        var selection2 = app.lib.getQueryParams(newQueryString) || {};
                        var selectItems = selection2["$select"] || "";
                        if (selectItems.length > 0) {
                            selectItems = selectItems.split(',');
                            columns = [];

                            for (var n in selectItems) {
                                var col = _.filter(config.allColumns,
                                    function (el) {
                                        return el.field === selectItems[n];
                                    });

                                if (col.length > 0) columns.push(col[0]);
                            }

                            var guidCol = _.filter(columns,
                                function (el) {
                                    return el.field === "Guid";
                                });

                            var guidColAll = _.filter(config.allColumns,
                                function (el) {
                                    return el.field === "Guid";
                                });

                            if (guidCol.length === 0 && guidColAll.length > 0) {
                                guidColAll[0].hidden = true;
                                newQueryString = "";
                                for (var key in selection2) {
                                    var val = selection2[key];
                                    if (key === "$select")
                                        val = val + ",Guid";
                                    newQueryString += (newQueryString.length > 0)
                                        ? "&" + key + "=" + val
                                        : "?" + key + "=" + val;
                                }
                                columns.push(guidColumnAll[0]);
                            }

                            vm.gridColumns = columns;
                        } else {
                            newQueryString = "";
                            for (var key in selection2) {
                                var val = selection2[key];
                                if (urlParams.hasOwnProperty(key) && key !== "$select") {
                                    newQueryString += (newQueryString.length > 0)
                                        ? "&" + key + "=" + val
                                        : "?" + key + "=" + val;
                                }
                            }
                        }
                    }

                    ODataGridConfigService.getGridConfig(configName).then(function (response) {

                        response.dataSource.transport.read.url = config.data_url + newQueryString + selectQuery;
                        vm.gridOptions = response;
                        vm.gridOptions.reorderable = false;
                        vm.gridOptions.columnMenu.columns = false;
                        vm.gridOptions.columns = columns;

                        if ($("#" + entity + "_tpl").html() && !config.isCustomGrid)
                            vm.gridOptions.rowTemplate =
                                $.proxy(kendo.template($("#" + entity + "_tpl").html()), vm);

                        if (config.isCustomGrid)
                            vm.gridOptions.rowTemplate = $.proxy(config.customGridTemplate, vm);

                        var fields = {};
                        angular.forEach(config.allColumns,
                            function (item) {
                                if (item.DataType == "date") {
                                    fields[item.field] = {
                                        type: item.DataType,
                                        parse: function (dateString) {
                                            if (_.isNull(dateString) || _.isUndefined(dateString)) {
                                                return '';
                                            } else {
                                                return new Date(dateString);
                                            }
                                        }
                                    };
                                } else
                                    fields[item.field] = { type: item.DataType };

                            });

                        vm.gridOptions.dataSource.schema = {
                            model: {
                                id: "id",
                                fields: fields
                            }
                        }
                    });

                },
                searchGrid: function (event) {
                    

                    if (event.which == "13" || !event) {

                        var data = $("#search_" + config.stateGuid).val();

                        var grid = $("div").find("[adf-grid-state-id='" + config.stateGuid + "']");

                        var dataSource = grid.data("kendoGrid").dataSource;
                        var dsFilters = (dataSource.filter()) ? dataSource.filter().filters : [];

                        if (data.length == 0) {

                            dsFilters = _.filter(dsFilters, function (el) {
                                return el.name != 'searchFilters';
                            });
                            $.each($("[adf-grid-state-id='" + config.stateGuid + "']" + " table th a.k-header-column-menu i"), function () {
                                $(this).remove();
                            });
                            grid.data("kendoGrid").dataSource.filter(dsFilters);

                        } else {

                            var current_url = dataSource.options.transport.read.url;
                            var queryStr = "";
                            if (current_url.indexOf('?') !== -1) {
                                queryStr = current_url.substr(current_url.indexOf('?'), current_url.length - 1);
                                var path = current_url.substr(0, current_url.indexOf('?'));

                                var queryParams = app.lib.getQueryParams(queryStr) || {};
                                queryStr = "";

                                if (queryParams.hasOwnProperty("search")) {
                                    for (var key in queryParams) {
                                        var val = queryParams[key];
                                        if (key === "search")
                                            val = data;
                                        queryStr += (queryStr.length > 0)
                                            ? "&" + key + "=" + val
                                            : "?" + key + "=" + val;
                                    }
                                    current_url = path + queryStr;
                                } else
                                    current_url += "&search=" + data;

                            } else 
                                current_url += "?search=" + data;

                            dataSource.options.transport.read.url = current_url;
                        }

                    }

                }
            });

            localizationService.getKendoMessageUrl().then(function(kendoMessageUrl) {
                $.getScript(kendoMessageUrl,
                    function() {
                        vm.getGridOptions();
                    });
            });
        }

        function ODataMobileGridController($scope, config, localizationService) {

            var newQueryString = config.queryString || '';
            var lang = (session.user.LanguageCode) ? session.user.LanguageCode : 'ENU';
            var urlParams = app.lib.getQueryParams() || {};

            if (newQueryString.length > 0) {

                _.each(config.queryParamKeys,
                    function (item) {
                        if (urlParams.hasOwnProperty(item.Key))
                            newQueryString = newQueryString.replace("{{" + item.Key + "}}", urlParams[item.Key]);
                        else
                            newQueryString = newQueryString.replace("{{" + item.Key + "}}", item.Value);
                    });

                if (newQueryString.indexOf('@User') !== -1) {
                    newQueryString = newQueryString.replace('@User', session.user.Id);
                }

                if (newQueryString.indexOf('@Lang') !== -1) {
                    newQueryString = newQueryString.replace('@Lang', lang);
                }
            }

            var columns = (config.gridColumns && config.gridColumns.length > 0) ? config.gridColumns : config.allColumns;
            var selectQuery = "";

            if (newQueryString.indexOf("$select") === -1) {
                if (columns && columns.length !== config.allColumns.length) {
                    var selection1 = _.map(columns,
                        function (el) {
                            return el.field;
                        });

                    selectQuery = "$select=" + selection1.toString();
                    selectQuery = (newQueryString.length > 0) ? "&" + selectQuery : "?" + selectQuery;
                }
            } else {
                var selection2 = app.lib.getQueryParams(newQueryString) || {};
                var selectItems = selection2["$select"];
                selectItems = selectItems.split(',');
                columns = [];

                for (var n in selectItems) {
                    var col = _.filter(config.allColumns,
                        function (el) {
                            return el.field === selectItems[n];
                        });

                    if (col.length > 0) columns.push(col[0]);
                }
            }

            var el = document.createElement('a');
            el.href = config.data_url;
            var path = el.pathname;
            var entity = path.substring(path.lastIndexOf("/") + 1, path.length);

            var fields = {};
            angular.forEach(config.allColumns, function (item) {
                if (item.DataType == "date") {
                    fields[item.field] = {
                        type: item.DataType,
                        parse: function (dateString) {
                            if (_.isNull(dateString) || _.isUndefined(dateString)) {
                                return '';
                            } else {
                                return kendo.toString(new Date(dateString), 'M/d/yyyy h:mm tt');
                            }
                        }
                    };
                } else
                    fields[item.field] = { type: item.DataType };

            });

            var vm = this;
            angular.extend(vm, {
                mobGridTemplate: (config.isCustomMobile) ? 'Custom' : (entity) ? entity : 'Default' ,
                gridStateId: config.stateGuid,
                dataSource: new kendo.data.DataSource({
                    type: "odata-v4",
                    transport: {
                        read: {
                            url: config.data_url + newQueryString + selectQuery,
                            dataType: "json"
                        }
                    },
                    pageSize: 10,
                    schema: {
                        model: {
                            id: "id",
                            fields: fields
                        }
                    },
                    serverPaging: true,
                    serverSorting: true,
                    serverFiltering: true
                }),
                columns: columns,
                pagerOptions: {
                    pageSizes: [10, 20, 50, 100],
                    pageSize: 10,
                    buttonCount: 1,
                    messages: {
                        display: localization.Viewing + " <strong>" + localization.Items.toLowerCase() + " {0}-{1}</strong> of {2} " + localization.Total + ".",
                        empty: "<span class='mobilegrid__pager--empty'>" + localization.NoResults + "</span>",
                        of: localization.Of,
                        first: localization.First,
                        previous: localization.Previous,
                        next: localization.Next,
                        last: localization.Last,
                        itemsPerPage: localization.ItemsPerPage,
                        allPages: localization.ViewingAllItems
                    }
                },
                dataBound: function (e) {
                    if (this.dataSource.view().length <= 0) {
                        //hide pager elements minus the info label
                        $('.mobilegrid__pager').children().not('.k-pager-info').hide();
                    } else {
                        //show pager
                        $('.mobilegrid__pager').children().show();
                    }
                },
                listViewOptions: {
                    dataBound: vm.dataBound
                }
            });

            var template = '';
            if (vm.mobGridTemplate != 'Default') {

                if (vm.mobGridTemplate == 'Custom')
                    template = config.customTemplate;
                else
                    template = kendo.template($("#" + vm.mobGridTemplate + "_mtpl").html());

                var dsFilters = vm.dataSource.filter() || [];

                var langCodeField = _.filter(config.allColumns, function (el) {
                    return el.field == "LanguageCode";
                });

                var langCodeFieldFilters = _.filter(dsFilters, function (el) {
                    return el.name == 'sysFilters';
                });

                if (langCodeField.length > 0 && langCodeFieldFilters.length == 0 && newQueryString.indexOf("LanguageCode") == -1) {
                    dsFilters = _.filter(dsFilters, function (el) {
                        return el.name != 'sysFilters';
                    });
                    dsFilters.push({
                        logic: 'or',
                        filters: [{
                            field: "LanguageCode",
                            operator: 'eq',
                            value: lang
                        }],
                        name: 'sysFilters'
                    });
                } else if (langCodeField.length > 0 && langCodeFieldFilters.length > 1 || newQueryString.indexOf("LanguageCode") > -1) {
                    dsFilters = _.filter(dsFilters, function (el) {
                        return el.name != 'sysFilters';
                    });
                }

                vm.dataSource.filter(dsFilters);

                setTimeout(function(){
                    $("#" + config.mobGridGUID).kendoListView({
                        dataSource: vm.dataSource,
                        template: template,
                        dataBound: vm.dataBound
                    });
                }, 1);
            }           
        }

        angular.module('adf.widget.odata.grid', ['adf.provider'])
            .config(["dashboardProvider", function (dashboardProvider) {
                dashboardProvider
                    .widget('odata-grid', {
                        title: localizationHelper.localize('AdfODataGridWidgetTitle', 'OData Table Widget'), //localization key
                        description: localizationHelper.localize('AdfODataGridWidgetDescription', 'Add a table based on an OData End Point'), //localization key
                        templateUrl: app.isMobile() ? '{widgetsPath}/odata-grid/src/viewMobile.html' : '{widgetsPath}/odata-grid/src/view.html',
                        controller: app.isMobile() ? 'ODataMobileGridController' : 'ODataGridController',
                        controllerAs: 'vm',
                        edit: {
                            templateUrl: '{widgetsPath}/odata-grid/src/edit.html',
                            controller: 'ODataGridEditController',
                            controllerAs: 'vm'
                        },
                        reload: true,
                        ordinal: 7
                    });
            }])
            .factory('ODataService', ['$http', '$q', 'NavigationNodeService', 'ViewPanelService', ODataService])
            .controller('ODataGridController', ['$scope', 'config', 'ODataGridConfigService', 'LocalizationService', ODataGridController])
            .controller('ODataMobileGridController', ['$scope', 'config', 'LocalizationService', ODataMobileGridController])
            .controller('ODataGridEditController', ['$scope', 'config', 'ODataService', 'ODataGridConfigService', 'LocalizationService', ODtataGridEditController]);

        angular.module("adf.widget.odata.grid").run(["$templateCache", function ($templateCache) {

            var editTemplate = ["<form role=form><div class=form-group>",
                "<label for=url>{{ ::vm.localize('ODataEndPoint') }}</label>",
                "<input type=text id=url ng-model=\"config.given_url\" class=\"form-control input-sm\" ng-change=\"vm.getCollections()\" />",
                "</div><div class=form-group>",
                "<label for=collections>{{ ::vm.localize('ChooseCollection') }}</label>",
                "<select id=collections class=\"form-control input-sm\" ng-model=\"config.gridCollection\" ng-change=\"vm.getColumns(config.given_url + '/' + config.gridCollection.field)\" ng-options=\"c.title for c in vm.collections track by c.field\">",
                "</select>",
                "</div><div class=form-group>",
                "<label for=queryString>{{ ::vm.localize('QueryString') }}</label>",
                "<input type=text id=queryString ng-model=\"config.queryString\" class=\"form-control input-sm\" ng-change=\"vm.onChangeQueryString()\"/>",
                "<p class='help-block' style='color: red' ng-if=\"vm.queryInvalid()\">{{ ::vm.localize('ODATAQueryInvalid') }}</p>",
                "</div>",
                "<div class=\"form-horizontal\">",
                    "<div class=form-group ng-if=\"config.queryParamKeys.length>0\">",
                        "<ul id=\"param-content\"><li ng-repeat=\"param in config.queryParamKeys track by $index\">",
                            "<div class=\"query-param\">",
                                "<label class=\"col-sm-3 control-label\">{{param.Key}}</label>",
                                "<div class=\"col-sm-9\"><input type=\"text\" class=\"k-textbox form-control input-sm \" ng-model=\"param.Value\"/></div>",
                            "</div>",
                        "</li></ul>",
                    "</div>",
                "</div>",
                "<div class=form-group>",
                "<label for=columns>{{ ::vm.localize('ChooseArrangeColumns') }}</label>",
                "<select id=columns class=\"form-control input-sm\" ng-model=\"gridColumns\">",
                "</select>",
                "<p class='help-block'>Drag items to change order.</p>",
                "</div><div class=form-group>",
                "<label>{{ ::vm.localize('AdvancedCustomization') }}</label>",
        			"<div class=checkbox checkbox-inline>",
                        "<input id=isCustomGrid name=isCustomGrid type=checkbox ng-model=\"config.isCustomGrid\" ng-change=\"vm.useCustomGrid()\"/>",
                        "<label class=control-label for=isCustomGrid ng-click=\"vm.isCustomGrid = !vm.isCustomGrid; config.customGridTemplate = ''\">",
                            "Custom Desktop Template",
                        "</label>",
                    "</div>",
                "</div><div class=form-group ng-if=\"config.isCustomGrid\">",
                "<textArea class=form-control id=customGridTemplate ng-model=config.customGridTemplate rows=3></textArea>",
                "</div><div class=form-group>",
        			"<div class=checkbox checkbox-inline>",
                        "<input id=isCustomMobile name=isCustomMobile type=checkbox ng-model=\"config.isCustomMobile\" ng-change=\"vm.useCustomMobile()\"/>",
                        "<label class=control-label for=isCustomMobile ng-click=\"vm.isCustomMobile = !vm.isCustomMobile; config.customTemplate = ''\">",
                            "{{ ::vm.localize('CustomMobileTemplate') }}",
                        "</label>",
                    "</div>",
                "</div><div class=form-group ng-if=\"config.isCustomMobile\">",
                "<textArea class=form-control id=customTemplate ng-model=config.customTemplate rows=3></textArea>",
                "</div></form>"].join("");
            $templateCache.put("{widgetsPath}/odata-grid/src/edit.html", editTemplate);

            var viewTemplate = [
                "<div class=\"clearfix\">",
                    "<div class=\"pull-left\">",
                    "<span class=\"odata-search-toolbar k-textbox k-space-right\">",
	                    "<input type=\"text\" id=\"{{'search_' + vm.gridStateId}}\" value=\"\" placeholder=\"Search...\" ng-keypress=\"vm.searchGrid($event)\">",
	                    "<a class=\"k-icon k-i-search ci-search\" ng-click=\"vm.searchGrid(false)\"></a>",
                    "</span>",
                    "</div>",
                    "<div class=\"pull-right\">",
                        "<a class=\"margin-l10 btn clear-filters-{{vm.gridStateId}}\">" + localization.ResetState + "</a>",
                    "</div>",
                "</div>",
                "<div adf-grid-state-id={{vm.gridStateId}} ",
                "kendo-grid=grid k-options=vm.gridOptions k-rebind=vm.gridOptions ",
                "k-on-column-reorder=vm.onReorderColumn(kendoEvent) ",
                "k-on-column-resize=vm.onColumnChange(kendoEvent) ",
                "k-on-column-show=vm.onColumnChange(kendoEvent) ",
                "k-on-column-hide=vm.onColumnChange(kendoEvent)>",
                "</div>"
            ].join("");
            $templateCache.put("{widgetsPath}/odata-grid/src/view.html", viewTemplate);

            var viewMobileTemplate = [
                "<div>",
                "<div class=\"mobilegrid__header\"></div>",
                "<div class=\"mobilegrid\" id=\"{{config.mobGridGUID}}\"></div>",
                "<div ng-if=\"vm.mobGridTemplate=='Default'\" kendo-list-view k-data-source=\"vm.dataSource\" k-options=\"vm.listViewOptions\" class=\"mobilegrid\">",
                    "<div class=\"gridcard\" k-template>",
                        "<p class=\"gridcard__title\">",
                        "<a> {{dataItem[vm.columns[0].field]}} </a>",
                        "</p>",
                        "<div class=\"gridcard__detailblock\">",
                        "<p ng-repeat=\"item in vm.columns\" ng-if=\"$index > 0\"> {{dataItem[item.field]}} </p>",
                "</div></div></div>",
                "<div class=\"mobilegrid__pager\" kendo-pager k-data-source=\"vm.dataSource\" k-options=\"vm.pagerOptions\"></div>",
                "</div>"
            ].join("");
            $templateCache.put("{widgetsPath}/odata-grid/src/viewMobile.html", viewMobileTemplate);
        }]);

        angular.module("adf.widget.odata.grid").filter("kendoFormatDate", function () {
            return function (date) {

                if (date)
                    return kendo.toString(new Date(date), 'M/d/yyyy h:mm tt');
                else return '';
            };
        });
    });
})();
