﻿/* global define: true */
/* global _: true */

(function () {
    'use strict';

    define(['app'], function (ngApp) {

        ngApp.factory('ODataGridConfigService', ['$http', '$q', 'LocalizationService', 'notificationService', ODataGridConfigService]);

        function ODataGridConfigService($http, $q, localizationService, notificationService) {

            var CONFIG_FILE_BASE_URL = '/Scripts/ng/page/adfWidgets/odataGrid/configFiles/';
            var JSON_FILE_EXT = '.js';
            var configMap = {};

            return {
                // getGridConfig: function(gridType) {
                // 	//NOTE: it is necessary to copy this object!
                // 	//if you do not, when the .dataSource is tacked on later
                // 	//by multiple grid controllers, only the last .dataSource
                // 	//will be bound to all grids (ie, all grids will display the same data)
                // 	var config = angular.copy(GRID_CONFIG[gridType]);
                // 	this.initConfigGridColumns(config);
                // 	return config;
                // },
                getConfigUrl: function (configName) {
                    var url = CONFIG_FILE_BASE_URL + configName + JSON_FILE_EXT;
                    return url;
                },
                getGridConfig: function (configName) {

                    var deferred = $q.defer();
                    var self = this;
                    // if (configMap[configName]) {
                    //     deferred.resolve(this.initConfig(configMap[configName]));
                    // } else {
                    var url = this.getConfigUrl(configName);
                    $http.get(url)
                        .then(function (response) {
                            var config = response.data;
                            config.dataSource.transport.read.beforeSend = function (e, request) {
                                var rewriteURL = self.rewriteFilterParams(request.url);
                                if (rewriteURL) request.url = rewriteURL;
                            };
                            configMap[configName] = config;
                            deferred.resolve(self.initConfig(config));
                        }, function () {
                            notificationService.addError("Could not retrieve config file");
                        });
                    // }

                    return deferred.promise;
                },
                rewriteFilterParams: function (original_url) {
                    var url = original_url;
                    var queryString = url.substr(url.indexOf('?'), url.length);
                    url = url.substr(0, url.indexOf('?')+1);
                    var params = this.getQueryParams(queryString)._wrapped || [];
                    var rewriteURL = url;
                    var filterCount = 0;
                    var filterStr = "$filter=";
                    var otherFiltersStr = "";
                    var topParam = _.find(params, function (item) {
                        return item[0] == "$top";
                    });
                    var bTopParamChange = false;

                    //set max record count to 500 if top is not defined. used to fix issue with exportToExcel function
                    if (_.isUndefined(topParam)) {
                        params.push({ 0: "$top", 1: "500" });
                        bTopParamChange = true;
                    }
                    
                    for (var i in params) {
                        if (params[i][0] == '$top') {
                            //set max record count to 500. used to fix issue with exportToExcel function
                            if (params[i][1] > 500) {
                                params[i][1] = 500;
                                bTopParamChange = true;
                            }
                        }

                        if (params[i][0] == '$filter') {
                            filterStr += params[i][1] + ' and '
                            filterCount++;
                        } else {
                            if (params[i][0].length > 0 && params[i][0] != '$format') {
                                params[i][1] = (params[i][1]) ? params[i][1] : '';
                                otherFiltersStr += params[i][0] + "=" + params[i][1] + "&";
                            }
                        }
                    }

                    if (filterStr == "$filter=") return original_url;

                    var checkEnd = filterStr.substr(filterStr.length - 5, filterStr.length);
                    if (checkEnd == ' and ')
                        filterStr = filterStr.substr(0, filterStr.length - 5);

                    if (filterCount > 1) {
                        if (otherFiltersStr.length > 0)
                            rewriteURL += filterStr + '&' + otherFiltersStr;
                        else
                            rewriteURL += filterStr;

                        return rewriteURL;
                    } else {
                        if (bTopParamChange) {
                            if (otherFiltersStr.length > 0)
                                rewriteURL += filterStr + '&' + otherFiltersStr;
                            else
                                rewriteURL += filterStr;
                            return rewriteURL;
                        }
                    }

                    return false;
                },
                getQueryParams: function (queryString) {
                    var query = (queryString || window.location.search).substring(1); // delete ?
                    if (!query) {
                        return false;
                    }
                    return _
                    .chain(query.split('&'))
                    .map(function (params) {
                        var p = params.split('=');
                        return [decodeURIComponent(p[0].toLowerCase()), p[1]];
                    })
                },
                getColumnConfig: function(entity){
                    var deferred = $q.defer();
                    $http.get('/Scripts/ng/page/adfWidgets/odataGrid/columnTemplates/' + entity + '.json')
                      .then(function (response) {
                          deferred.resolve(response.data);
                      }, function () {
                          console.error("Could not retrieve column config file");
                          deferred.resolve({});
                      });

                    return deferred.promise;

                },
                initConfig: function (config) {
                    this.initConfigGridColumns(config);
                    this.initPageableMessages(config);
                    this.initFilterableMessages(config);
                    this.initGroupableMessages(config);
                    this.initColumnMenu(config);
                    this.initEventBinding(config);
                    return angular.copy(config);
                },
                initEventBinding: function (config) {
                    config.dataBound = function (e) {
                        var grid = e.sender;
                        var gridId = e.sender.$angular_scope.stateGuid;
                        var currentState = app.gridUtils.savedState.getCurrentState(gridId);

                        app.gridUtils.denoteAppliedFilters(grid, currentState);
                        app.gridUtils.setDashboardGridState(gridId, grid, grid.dataSource, currentState);
                        app.gridUtils.initDashboardRowClickHandling(gridId, grid);
                        app.gridUtils.handleEmptyResults(grid, grid.dataSource, currentState);
                        app.events.publish('gridBound', grid);
                    };
                },
                initColumnMenu: function (config) {
                    if (config.columnMenu && config.columnMenu.messages) {
                        angular.extend(config.columnMenu.messages, {
                            columns: localizationService.getValue('ChooseColumns'),
                            filter: localizationService.getValue('Filter'),
                            sortAscending: localizationService.getValue('SortAscending'),
                            sortDescending: localizationService.getValue('SortDescending')
                        });
                    }
                },
                initConfigGridColumns: function (config) {
                    var self = this;
                    _.each(config.columns, self.initGridColumn);
                },
                initFilterableMessages: function (config) {

                    if (config.filterable) {

                        if (config.filterable.messages) {
                            angular.extend(config.filterable.messages, {
                                info: localizationService.getValue('Showitemswithvaluethat'),
                                and: localizationService.getValue('And'),
                                or: localizationService.getValue('Or'),
                                filter: localizationService.getValue('Filter'),
                                clear: localizationService.getValue('Clear')
                            });
                        }

                        if (config.filterable.operators) {

                            if (config.filterable.operators.string) {
                                angular.extend(config.filterable.operators.string, {
                                    eq: localizationService.getValue('Isequalto'),
                                    neq: localizationService.getValue('Isnotequalto'),
                                    contains: localizationService.getValue('Contains'),
                                    doesnotcontain: localizationService.getValue('DoesNotContain'),
                                    startswith: localizationService.getValue('Startswith'),
                                    endswith: localizationService.getValue('Endswith')
                                });
                            }

                            if (config.filterable.operators.number) {
                                angular.extend(config.filterable.operators.number, {
                                    eq: localizationService.getValue('Isequalto'),
                                    neq: localizationService.getValue('Isnotequalto'),
                                    gt: localizationService.getValue('GreaterThan'),
                                    gte: localizationService.getValue('GreaterOrEqual'),
                                    lt: localizationService.getValue('LessThan'),
                                    lte: localizationService.getValue('LessOrEqual'),
                                });
                            }

                            if (config.filterable.operators.SLOStatus) {
                                angular.extend(config.filterable.operators.SLOStatus, {
                                    contains: localizationService.getValue('Contains')
                                });
                            }
                        }
                    }

                },
                initGridColumn: function (column) {
                    if (!column.title) {
                        column.title = column.field;
                    }
                },
                initGroupableMessages: function (config) {
                    if (config.groupable && config.groupable.messages) {
                        angular.extend(config.groupable.messages, {
                            empty: localizationService.getValue('Dragacolumnheader')
                        });
                    }
                },
                initPageableMessages: function (config) {
                    if (config.pageable && config.pageable.messages) {
                        angular.extend(config.pageable.messages, {

                            empty: localizationService.getValue('Empty'),
                            of: localizationService.getValue('Of'),
                            first: localizationService.getValue('First'),
                            previous: localizationService.getValue('Previous'),
                            next: localizationService.getValue('Next'),
                            last: localizationService.getValue('Last'),
                            itemsPerPage: localizationService.getValue('ItemsPerPage')
                        });
                    }
                },
                setErrorNotification: function (message) {
                    notificationService.addError(message);
                },
                guid: function(){
                    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
                },
                getGuid: function () {
                    var guid = (this.guid() + this.guid() + "-" + this.guid() + "-" + this.guid() + "-" +
this.guid() + "-" + this.guid() + this.guid() + this.guid()).toUpperCase();

                    return guid;
                }
            };
        }
    });
})();
