﻿set-location $PSScriptroot
$pctComplete=0
$LogFilePath = (${env:Temp} + "\CiresonInstall" + (Get-Date -format "yyyy-MM-dd-hh-mm-ss") + ".log")

function writeToLog([string] $message)
{
    $message + "`n"
    Add-Content $LogFilePath ((Get-Date -format "yyyy-MM-dd-hh-mm-ss") + ": " + $message + "`n")
}

function writeProgress([int] $progress)
{
    ("%progress%=" + $progress+ "`n")
}


function writeWarning([string] $warning)
{
	writeToLog warning;
    ("%warning%=" + $warning + "`n")
}

function installRequiredWindowsFeatures(){

    writeToLog "Enabling required windows features."
    #Install prereq features.
    $features = @("Web-Server", "Web-Mgmt-Tools", "web-windows-auth", "Application-Server", "Web-Asp-Net45", "NET-Framework-Features", "Web-Static-Content")

    foreach($feature in $features)
    {
        try
        {
			$pctComplete = $pctComplete + 10;
			writeProgress $pctComplete;
            $srv=Get-WindowsFeature $feature
            if(! $srv.Installed){
                writeToLog ("installing feature: " + $feature)
                install-WindowsFeature $feature
            }
            writeToLog ($feature + " enabled.")
        }catch [Exception]{
            $ex = $_.Exception.Message
            writeWarning ("Failed to install windows feature: " + $ex)
        }
    }
}

function installRequiredPrereqs(){
    try{
        $msiFiles = Get-ChildItem "InstallationFiles\MSI" -Filter *.msi | sort-object Name
    
        foreach($msi in $msiFiles)
        {
			$pctComplete = $pctComplete + 5
			writeProgress $pctComplete;
            writeToLog ("Installing " + $msi.Name)
            Start-Process $msi.FullName /qn -Wait        
        }
    }catch [Exception]{
        $ex = $_.Exception.Message
        writeWarning ("Failed to install prereqs: " + $ex)
    }
}


#Enable required windows features.
installRequiredWindowsFeatures
#install prereq i packages.
installRequiredPrereqs
$pctComplete = 100

writeWarning("Please restart your server to complete the installation.")