--*****************************************
-- Name: Create Data Grooming Job for DW Lite  
--
-- Desc: Creates the grooming SQL Agent jobs 
--       that deletes aged works items in 
--       tables in DW lite solution. 
--
--*****************************************

USE [msdb]
GO

/****** Object:  Job [Cireson Delete Aged Work Items]    Script Date: 3/26/2016 9:34:42 AM ******/
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

--define the database
DECLARE @database NVARCHAR(MAX)
-- Change %AnalyticDBName% into CiresonAnalytics for debugging
SET @database = N'%AnalyticDBName%'

--get the db owner
DECLARE @dbowner NVARCHAR(MAX)
BEGIN
SELECT @dbowner = suser_sname(owner_sid)
  FROM sys.databases
 WHERE name=@database
END


/****** Object:  JobCategory [[Uncategorized (Local)]]    Script Date: 3/26/2016 9:34:42 AM ******/
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'[Uncategorized (Local)]' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'[Uncategorized (Local)]'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'Cireson Delete Aged Work Items', 
		@enabled=1, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Deletes work items older than the age threshold (in days) set in the SM_Data_Retention table in the ServiceManagement database.', 
		@category_name=N'[Uncategorized (Local)]', 
		@owner_login_name=@dbowner, @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [Cireson Delete Aged Work Items]    Script Date: 3/26/2016 9:34:43 AM ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Cireson Delete Aged Work Items', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'
--*******************************************************
-- Name: Cireson Delete Aged Work Items 
--
-- Desc: Deletes work items older than data retention 
--       threshold defined in the SM_Data_Retention table 
--
--*******************************************************

--*********************
--Delete Old Incidents 
--*********************
Declare @IncidentAgeDays as INT 
SET @IncidentAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_Incident'')
--PRINT @IncidentAgeDays

DELETE FROM SM_WorkItem_Incident
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @IncidentAgeDays

--*********************
--Delete Old Problems 
--*********************
Declare @ProblemAgeDays as INT 
SET @ProblemAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_Problem'')
--PRINT @ProblemAgeDays

DELETE FROM SM_WorkItem_Problem
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ProblemAgeDays

--*********************
--Delete Old Service Requests 
--*********************
Declare @ServiceRequestAgeDays as INT 
SET @ServiceRequestAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_ServiceRequest'')
--PRINT @ServiceRequestAgeDays

DELETE FROM SM_WorkItem_ServiceRequest
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ServiceRequestAgeDays

--*********************
--Delete Old Changes 
--*********************
Declare @ChangeAgeDays as INT 
SET @ChangeAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_Change'')
--PRINT @ChangeAgeDays

DELETE FROM SM_WorkItem_Change
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ChangeAgeDays

--*********************
--Delete Old Releases 
--*********************
Declare @ReleaseAgeDays as INT 
SET @ReleaseAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_Release'')
--PRINT @ReleaseAgeDays 

DELETE FROM SM_WorkItem_Release
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ReleaseAgeDays

--*********************
--Delete Old Manual Activities 
--*********************
Declare @ManualActivityAgeDays as INT 
SET @ManualActivityAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_ManualActivity'')
--PRINT @ManualActivityAgeDays 

DELETE FROM SM_WorkItem_Release
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ReleaseAgeDays

--*********************
--Delete Old Review Activites 
--*********************
Declare @ReviewActivityAgeDays as INT 
SET @ReviewActivityAgeDays = (SELECT Top 1 Data_Retention_Days FROM SM_Data_Retention WHERE Data_Item_Type LIKE ''WI_ReviewActivity'')
--PRINT @ReviewActivityAgeDays 

DELETE FROM SM_WorkItem_ReviewActivity
WHERE DATEDIFF(DD,LastModified,GETDATE()) > @ReviewActivityAgeDays', 
		@database_name=@database, 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Cireson Delete Aged Work Items', 
		@enabled=1, 
		@freq_type=8, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=N'%AnalyticFrequency%', 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=1, 
		@active_start_date=N'%AnalyticFrequencyStartDate%', 
		@active_end_date=99991231, 
		@active_start_time=N'%AnalyticFrequencyStartTime%', 
		@active_end_time=235959, 
		@schedule_uid=N'0c6eeb93-0cbe-432f-8758-688d3f920201'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO


